@@ -38,14 +38,20 @@
 
 #include "nsXULSliderAccessible.h"
 
+#include "nsAccessibilityAtoms.h"
+#include "States.h"
+
 #include "nsIDOMDocument.h"
 #include "nsIDOMDocumentXBL.h"
+#include "nsIFrame.h"
 
+////////////////////////////////////////////////////////////////////////////////
 // nsXULSliderAccessible
+////////////////////////////////////////////////////////////////////////////////
 
-nsXULSliderAccessible::nsXULSliderAccessible(nsIDOMNode* aNode,
-                                             nsIWeakReference* aShell) :
-  nsAccessibleWrap(aNode, aShell)
+nsXULSliderAccessible::
+  nsXULSliderAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsAccessibleWrap(aContent, aShell)
 {
 }
 
@@ -57,34 +63,28 @@ NS_IMPL_ISUPPORTS_INHERITED1(nsXULSliderAccessible,
 
 // nsAccessible
 
-nsresult
-nsXULSliderAccessible::GetRoleInternal(PRUint32 *aRole)
+PRUint32
+nsXULSliderAccessible::NativeRole()
 {
-  *aRole = nsIAccessibleRole::ROLE_SLIDER;
-  return NS_OK;
+  return nsIAccessibleRole::ROLE_SLIDER;
 }
 
-nsresult
-nsXULSliderAccessible::GetStateInternal(PRUint32 *aState,
-                                        PRUint32 *aExtraState)
+PRUint64
+nsXULSliderAccessible::NativeState()
 {
-  nsresult rv = nsAccessibleWrap::GetStateInternal(aState, aExtraState);
-  NS_ENSURE_A11Y_SUCCESS(rv, rv);
+  PRUint64 states = nsAccessibleWrap::NativeState();
 
   nsCOMPtr<nsIContent> sliderContent(GetSliderNode());
   NS_ENSURE_STATE(sliderContent);
 
-  nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mWeakShell));
-  NS_ENSURE_STATE(shell);
-
-  nsIFrame *frame = shell->GetPrimaryFrameFor(sliderContent);
+  nsIFrame *frame = sliderContent->GetPrimaryFrame();
   if (frame && frame->IsFocusable())
-    *aState |= nsIAccessibleStates::STATE_FOCUSABLE;
+    states |= states::FOCUSABLE;
 
-  if (gLastFocusedNode == mDOMNode)
-    *aState |= nsIAccessibleStates::STATE_FOCUSED;
+  if (gLastFocusedNode == mContent)
+    states |= states::FOCUSED;
 
-  return NS_OK;
+  return states;
 }
 
 // nsIAccessible
@@ -123,7 +123,8 @@ nsXULSliderAccessible::DoAction(PRUint8 aIndex)
   nsCOMPtr<nsIContent> sliderContent(GetSliderNode());
   NS_ENSURE_STATE(sliderContent);
 
-  return DoCommand(sliderContent);
+  DoCommand(sliderContent);
+  return NS_OK;
 }
 
 // nsIAccessibleValue
@@ -200,12 +201,11 @@ nsXULSliderAccessible::GetAllowsAnonChildAccessibles()
 already_AddRefed<nsIContent>
 nsXULSliderAccessible::GetSliderNode()
 {
-  if (!mDOMNode)
+  if (IsDefunct())
     return nsnull;
 
   if (!mSliderNode) {
-    nsCOMPtr<nsIDOMDocument> document;
-    mDOMNode->GetOwnerDocument(getter_AddRefs(document));
+    nsIDocument* document = mContent->GetOwnerDoc();
     if (!document)
       return nsnull;
 
@@ -214,7 +214,7 @@ nsXULSliderAccessible::GetSliderNode()
       return nsnull;
 
     // XXX: we depend on anonymous content.
-    nsCOMPtr<nsIDOMElement> domElm(do_QueryInterface(mDOMNode));
+    nsCOMPtr<nsIDOMElement> domElm(do_QueryInterface(mContent));
     if (!domElm)
       return nsnull;
 
@@ -271,7 +271,7 @@ nsXULSliderAccessible::GetSliderAttr(nsIAtom *aName, double *aValue)
     return NS_OK;
 
   PRInt32 error = NS_OK;
-  double value = attrValue.ToFloat(&error);
+  double value = attrValue.ToDouble(&error);
   if (NS_SUCCEEDED(error))
     *aValue = value;
 
@@ -288,18 +288,22 @@ nsXULSliderAccessible::SetSliderAttr(nsIAtom *aName, double aValue)
 }
 
 
+////////////////////////////////////////////////////////////////////////////////
 // nsXULThumbAccessible
+////////////////////////////////////////////////////////////////////////////////
 
-nsXULThumbAccessible::nsXULThumbAccessible(nsIDOMNode* aNode,
-                                           nsIWeakReference* aShell) :
-  nsAccessibleWrap(aNode, aShell) {}
-
-// nsIAccessible
-
-nsresult
-nsXULThumbAccessible::GetRoleInternal(PRUint32 *aRole)
+nsXULThumbAccessible::
+  nsXULThumbAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsAccessibleWrap(aContent, aShell)
 {
-  *aRole = nsIAccessibleRole::ROLE_INDICATOR;
-  return NS_OK;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// nsXULThumbAccessible: nsAccessible
+
+PRUint32
+nsXULThumbAccessible::NativeRole()
+{
+  return nsIAccessibleRole::ROLE_INDICATOR;
 }
 
