@@ -116,6 +116,13 @@ function scorePatternMatch(pattern, matched, offset) {
 // <TabItem>s and <xul:tab>s without having to worry which
 // one is which.
 var TabUtils = {
+  // ----------
+  // Function: toString
+  // Prints [TabUtils] for debug use
+  toString: function TabUtils_toString() {
+    return "[TabUtils]";
+  },
+
   // ---------
   // Function: _nameOfTab
   // Given a <TabItem> or a <xul:tab> returns the tab's name.
@@ -125,7 +132,7 @@ var TabUtils = {
     // of active Panoramas as well as for windows in which
     // Panorama has yet to be activated. We uses object sniffing to
     // determine the type of tab and then returns its name.     
-    return tab.label != undefined ? tab.label : tab.$tabTitle[0].innerHTML;
+    return tab.label != undefined ? tab.label : tab.$tabTitle[0].textContent;
   },
   
   // ---------
@@ -166,7 +173,14 @@ function TabMatcher(term) {
   this.term = term; 
 }
 
-TabMatcher.prototype = {  
+TabMatcher.prototype = {
+  // ----------
+  // Function: toString
+  // Prints [TabMatcher (term)] for debug use
+  toString: function TabMatcher_toString() {
+    return "[TabMatcher (" + this.term + ")]";
+  },
+
   // ---------
   // Function: _filterAndSortMatches
   // Given an array of <TabItem>s and <xul:tab>s returns a new array
@@ -196,7 +210,7 @@ TabMatcher.prototype = {
   _filterForUnmatches: function TabMatcher__filterForUnmatches(tabs) {
     var self = this;
     return tabs.filter(function(tab) {
-      var name = tab.$tabTitle[0].innerHTML;
+      let name = tab.$tabTitle[0].textContent;
       let url = TabUtils.URLOf(tab);
       return !name.match(self.term, "i") && !url.match(self.term, "i");
     });
@@ -204,43 +218,30 @@ TabMatcher.prototype = {
   
   // ---------
   // Function: _getTabsForOtherWindows
-  // Returns an array of <TabItem>s and <xul:tabs>s representing that
-  // tabs from all windows but the currently focused window. <TabItem>s
-  // will be returned for windows in which Panorama has been activated at
-  // least once, while <xul:tab>s will be return for windows in which
-  // Panorama has never been activated.
-  _getTabsForOtherWindows: function TabMatcher__getTabsForOtherWindows(){
-    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
-                       .getService(Components.interfaces.nsIWindowMediator);
-    var enumerator = wm.getEnumerator("navigator:browser");    
-    var currentWindow = wm.getMostRecentWindow("navigator:browser");
-    
-    var allTabs = [];    
+  // Returns an array of <TabItem>s and <xul:tabs>s representing tabs
+  // from all windows but the current window. <TabItem>s will be returned
+  // for windows in which Panorama has been activated at least once, while
+  // <xul:tab>s will be returned for windows in which Panorama has never
+  // been activated.
+  _getTabsForOtherWindows: function TabMatcher__getTabsForOtherWindows() {
+    var enumerator = Services.wm.getEnumerator("navigator:browser");
+    var allTabs = [];
+
     while (enumerator.hasMoreElements()) {
       var win = enumerator.getNext();
-      // This function gets tabs from other windows: not the one you currently
-      // have focused.
-      if (win != currentWindow) {
-        // If TabView is around iterate over all tabs, else get the currently
-        // shown tabs...
-        
-        let tvWindow = win.TabView.getContentWindow();
-        if (tvWindow)
-          allTabs = allTabs.concat( tvWindow.TabItems.getItems() );
-        else
-          // win.gBrowser.tabs isn't a proper array, so we can't use concat
-          for (var i=0; i<win.gBrowser.tabs.length; i++) allTabs.push( win.gBrowser.tabs[i] );
-      } 
+      // This function gets tabs from other windows, not from the current window
+      if (win != gWindow)
+        allTabs.push.apply(allTabs, win.gBrowser.tabs);
     }
-    return allTabs;    
+    return allTabs;
   },
   
   // ----------
   // Function: matchedTabsFromOtherWindows
   // Returns an array of <TabItem>s and <xul:tab>s that match the search term
-  // from all windows but the currently focused window. <TabItem>s will be
-  // returned for windows in which Panorama has been activated at least once,
-  // while <xul:tab>s will be return for windows in which Panorama has never
+  // from all windows but the current window. <TabItem>s will be returned for
+  // windows in which Panorama has been activated at least once, while
+  // <xul:tab>s will be returned for windows in which Panorama has never
   // been activated.
   // (new TabMatcher("app")).matchedTabsFromOtherWindows();
   matchedTabsFromOtherWindows: function TabMatcher_matchedTabsFromOtherWindows(){
@@ -316,14 +317,20 @@ function SearchEventHandlerClass() {
 }
 
 SearchEventHandlerClass.prototype = {
+  // ----------
+  // Function: toString
+  // Prints [SearchEventHandler] for debug use
+  toString: function SearchEventHandlerClass_toString() {
+    return "[SearchEventHandler]";
+  },
+
   // ----------
   // Function: init
   // Initializes the searchbox to be focused, and everything
   // else to be hidden, and to have everything have the appropriate
   // event handlers;
   init: function () {
-    var self = this;
-    iQ("#searchbox")[0].focus(); 
+    let self = this;
     iQ("#search").hide();
     iQ("#searchshade").hide().click(function(event) {
       if ( event.target.id != "searchbox")
@@ -369,6 +376,13 @@ SearchEventHandlerClass.prototype = {
     if (event.target.nodeName == "INPUT")
       return;
 
+    // / is used to activate the search feature so the key shouldn't be entered 
+    // into the search box.
+    if (event.keyCode == KeyEvent.DOM_VK_SLASH) {
+      event.stopPropagation();
+      event.preventDefault();
+    }
+
     this.switchToInMode();
     this.initiatedBy = "keydown";
     ensureSearchShown(true);
@@ -505,7 +519,7 @@ function createSearchTabMacher() {
   return new TabMatcher(iQ("#searchbox").val());
 }
 
-function hideSearch(event){
+function hideSearch(event) {
   iQ("#searchbox").val("");
   iQ("#searchshade").hide();
   iQ("#search").hide();
@@ -519,7 +533,12 @@ function hideSearch(event){
   performSearch();
   SearchEventHandler.switchToBeforeMode();
 
-  if (event){
+  if (event) {
+    // when hiding the search mode, we need to prevent the keypress handler
+    // in UI__setTabViewFrameKeyHandlers to handle the key press again. e.g. Esc
+    // which is already handled by the key down in this class.
+    if (event.type == "keydown")
+      UI.ignoreKeypressForSearch = true;
     event.preventDefault();
     event.stopPropagation();
   }
