@@ -44,7 +44,6 @@
 #include "nsFormSubmission.h"
 #include "nsGenericHTMLElement.h"
 #include "nsIDOMHTMLFormElement.h"
-#include "nsIDOMNSHTMLFormElement.h"
 #include "nsIWebProgressListener.h"
 #include "nsIRadioGroupContainer.h"
 #include "nsIURI.h"
@@ -93,7 +92,6 @@ private:
 
 class nsHTMLFormElement : public nsGenericHTMLElement,
                           public nsIDOMHTMLFormElement,
-                          public nsIDOMNSHTMLFormElement,
                           public nsIWebProgressListener,
                           public nsIForm,
                           public nsIRadioGroupContainer_MOZILLA_2_0_BRANCH
@@ -119,9 +117,6 @@ public:
   // nsIDOMHTMLFormElement
   NS_DECL_NSIDOMHTMLFORMELEMENT
 
-  // nsIDOMNSHTMLFormElement
-  NS_DECL_NSIDOMNSHTMLFORMELEMENT  
-
   // nsIWebProgressListener
   NS_DECL_NSIWEBPROGRESSLISTENER
 
