@@ -44,7 +44,7 @@
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsCRT.h"
 #include "nsString.h"
-#include "nsContentUtils.h"
+#include "mozilla/Preferences.h"
 
 #include "nsIControllerCommandTable.h"
 #include "nsICommandParams.h"
@@ -64,6 +64,8 @@
 #include "nsIClipboardDragDropHooks.h"
 #include "nsIClipboardDragDropHookList.h"
 
+using namespace mozilla;
+
 const char * const sSelectAllString = "cmd_selectAll";
 const char * const sSelectNoneString = "cmd_selectNone";
 const char * const sCopyImageLocationString = "cmd_copyImageLocation";
@@ -246,7 +248,7 @@ nsSelectMoveScrollCommand::DoSelectCommand(const char *aCommandName, nsIDOMWindo
   PRBool caretOn = PR_FALSE;
   selCont->GetCaretEnabled(&caretOn);
   if (!caretOn) {
-    caretOn = nsContentUtils::GetBoolPref("accessibility.browsewithcaret");
+    caretOn = Preferences::GetBool("accessibility.browsewithcaret");
     if (caretOn) {
       nsCOMPtr<nsPIDOMWindow> piWindow = do_QueryInterface(aWindow);
       if (piWindow) {
