@@ -853,9 +853,8 @@ NS_IMPL_ISUPPORTS_INHERITED3(nsDOMWorkerScope, nsDOMWorkerMessageHandler,
                                                nsIWorkerGlobalScope,
                                                nsIXPCScriptable)
 
-NS_IMPL_CI_INTERFACE_GETTER5(nsDOMWorkerScope, nsIWorkerScope,
+NS_IMPL_CI_INTERFACE_GETTER4(nsDOMWorkerScope, nsIWorkerScope,
                                                nsIWorkerGlobalScope,
-                                               nsIDOMNSEventTarget,
                                                nsIDOMEventTarget,
                                                nsIXPCScriptable)
 
@@ -914,7 +913,6 @@ nsDOMWorkerScope::Trace(nsIXPConnectWrappedNative* /* aWrapper */,
                         JSTracer* aTracer,
                         JSObject* /*aObj */)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   nsDOMWorkerMessageHandler::Trace(aTracer);
   return NS_OK;
 }
@@ -1159,14 +1157,6 @@ nsDOMWorkerScope::SetOnclose(nsIDOMEventListener* aOnclose)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsDOMWorkerScope::AddEventListener(const nsAString& aType,
-                                   nsIDOMEventListener* aListener,
-                                   PRBool aUseCapture)
-{
-  return AddEventListener(aType, aListener, aUseCapture, PR_FALSE, 1);
-}
-
 NS_IMETHODIMP
 nsDOMWorkerScope::RemoveEventListener(const nsAString& aType,
                                       nsIDOMEventListener* aListener,
@@ -1440,9 +1430,8 @@ NS_INTERFACE_MAP_BEGIN(nsDOMWorker)
   NS_INTERFACE_MAP_ENTRY(nsIXPCScriptable)
   NS_INTERFACE_MAP_ENTRY(nsIWorker)
   NS_INTERFACE_MAP_ENTRY(nsIAbstractWorker)
-  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIDOMNSEventTarget,
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIDOMEventTarget,
                                    nsDOMWorkerMessageHandler)
-  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIDOMEventTarget, nsDOMWorkerMessageHandler)
   NS_INTERFACE_MAP_ENTRY(nsIJSNativeInitializer)
   NS_INTERFACE_MAP_ENTRY(nsITimerCallback)
 NS_INTERFACE_MAP_END
@@ -1509,8 +1498,6 @@ nsDOMWorker::Trace(nsIXPConnectWrappedNative* /* aWrapper */,
                    JSTracer* aTracer,
                    JSObject* /*aObj */)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-
   PRBool canceled = PR_FALSE;
   {
     MutexAutoLock lock(mLock);
@@ -1550,9 +1537,8 @@ nsDOMWorker::Finalize(nsIXPConnectWrappedNative* /* aWrapper */,
   return NS_OK;
 }
 
-NS_IMPL_CI_INTERFACE_GETTER4(nsDOMWorker, nsIWorker,
+NS_IMPL_CI_INTERFACE_GETTER3(nsDOMWorker, nsIWorker,
                                           nsIAbstractWorker,
-                                          nsIDOMNSEventTarget,
                                           nsIDOMEventTarget)
 NS_IMPL_THREADSAFE_DOM_CI_GETINTERFACES(nsDOMWorker)
 NS_IMPL_THREADSAFE_DOM_CI_ALL_THE_REST(nsDOMWorker)
@@ -2564,14 +2550,6 @@ nsDOMWorker::QueueSuspendedRunnable(nsIRunnable* aRunnable)
   return mQueuedRunnables.AppendElement(aRunnable) ? PR_TRUE : PR_FALSE;
 }
 
-NS_IMETHODIMP
-nsDOMWorker::AddEventListener(const nsAString& aType,
-                              nsIDOMEventListener* aListener,
-                              PRBool aUseCapture)
-{
-  return AddEventListener(aType, aListener, aUseCapture, PR_FALSE, 1);
-}
-
 NS_IMETHODIMP
 nsDOMWorker::RemoveEventListener(const nsAString& aType,
                                  nsIDOMEventListener* aListener,
@@ -2611,7 +2589,7 @@ nsDOMWorker::AddEventListener(const nsAString& aType,
                               nsIDOMEventListener* aListener,
                               PRBool aUseCapture,
                               PRBool aWantsUntrusted,
-                              PRUint8 optional_argc)
+                              PRUint8 aOptionalArgc)
 {
   NS_ASSERTION(mWrappedNative, "Called after Finalize!");
   if (IsCanceled()) {
@@ -2621,7 +2599,7 @@ nsDOMWorker::AddEventListener(const nsAString& aType,
   return nsDOMWorkerMessageHandler::AddEventListener(aType, aListener,
                                                      aUseCapture,
                                                      aWantsUntrusted,
-                                                     optional_argc);
+                                                     aOptionalArgc);
 }
 
 /**
