@@ -36,14 +36,8 @@
 
 
 #include "mozilla/ModuleUtils.h"
-
-#ifdef MOZ_MACBROWSER
-#include "mozOSXSpell.h"
-#else
 #include "mozHunspell.h"
 #include "mozHunspellDirProvider.h"
-#endif
-
 #include "mozSpellChecker.h"
 #include "mozInlineSpellChecker.h"
 #include "nsTextServicesCID.h"
@@ -60,13 +54,8 @@
 0x9fe5d975, 0x9bd, 0x44aa,                      \
 { 0xa0, 0x1a, 0x66, 0x40, 0x2e, 0xa2, 0x86, 0x57} }
 
-#ifdef MOZ_MACBROWSER
-NS_GENERIC_FACTORY_CONSTRUCTOR(mozOSXSpell)
-#else
 NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(mozHunspell, Init)
 NS_GENERIC_FACTORY_CONSTRUCTOR(mozHunspellDirProvider)
-#endif
-
 NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(mozSpellChecker, Init)
 NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(mozPersonalDictionary, Init)
 NS_GENERIC_FACTORY_CONSTRUCTOR(mozSpellI18NManager)
@@ -104,24 +93,16 @@ mozInlineSpellCheckerConstructor(nsISupports *aOuter, REFNSIID aIID,
   return rv;
 }
 
-#ifdef MOZ_MACBROWSER
-NS_DEFINE_NAMED_CID(MOZ_OSXSPELL_CID);
-#else
 NS_DEFINE_NAMED_CID(MOZ_HUNSPELL_CID);
 NS_DEFINE_NAMED_CID(HUNSPELLDIRPROVIDER_CID);
-#endif // MOZ_MACBROWSER
 NS_DEFINE_NAMED_CID(NS_SPELLCHECKER_CID);
 NS_DEFINE_NAMED_CID(MOZ_PERSONALDICTIONARY_CID);
 NS_DEFINE_NAMED_CID(MOZ_SPELLI18NMANAGER_CID);
 NS_DEFINE_NAMED_CID(MOZ_INLINESPELLCHECKER_CID);
 
 static const mozilla::Module::CIDEntry kSpellcheckCIDs[] = {
-#ifdef MOZ_MACBROWSER
-    { &kMOZ_OSXSPELL_CID, false, NULL, mozOSXSpellConstructor },
-#else
     { &kMOZ_HUNSPELL_CID, false, NULL, mozHunspellConstructor },
     { &kHUNSPELLDIRPROVIDER_CID, false, NULL, mozHunspellDirProviderConstructor },
-#endif // MOZ_MACBROWSER
     { &kNS_SPELLCHECKER_CID, false, NULL, mozSpellCheckerConstructor },
     { &kMOZ_PERSONALDICTIONARY_CID, false, NULL, mozPersonalDictionaryConstructor },
     { &kMOZ_SPELLI18NMANAGER_CID, false, NULL, mozSpellI18NManagerConstructor },
@@ -130,12 +111,8 @@ static const mozilla::Module::CIDEntry kSpellcheckCIDs[] = {
 };
 
 static const mozilla::Module::ContractIDEntry kSpellcheckContracts[] = {
-#ifdef MOZ_MACBROWSER
-    { MOZ_OSXSPELL_CONTRACTID, &kMOZ_OSXSPELL_CID },
-#else
     { MOZ_HUNSPELL_CONTRACTID, &kMOZ_HUNSPELL_CID },
     { mozHunspellDirProvider::kContractID, &kHUNSPELLDIRPROVIDER_CID },
-#endif // MOZ_MACBROWSER
     { NS_SPELLCHECKER_CONTRACTID, &kNS_SPELLCHECKER_CID },
     { MOZ_PERSONALDICTIONARY_CONTRACTID, &kMOZ_PERSONALDICTIONARY_CID },
     { MOZ_SPELLI18NMANAGER_CONTRACTID, &kMOZ_SPELLI18NMANAGER_CID },
@@ -144,9 +121,7 @@ static const mozilla::Module::ContractIDEntry kSpellcheckContracts[] = {
 };
 
 static const mozilla::Module::CategoryEntry kSpellcheckCategories[] = {
-#ifndef MOZ_MACBROWSER
     { XPCOM_DIRECTORY_PROVIDER_CATEGORY, "spellcheck-directory-provider", mozHunspellDirProvider::kContractID },
-#endif
     { NULL }
 };
 
