@@ -71,6 +71,9 @@ class Visitor:
     def visitTypedef(self, tdef):
         tdef.fromtype.accept(self)
 
+    def visitUsing(self, us):
+        us.type.accept(self)
+
     def visitForwardDecl(self, fd):
         pass
 
@@ -98,11 +101,21 @@ class Visitor:
             param.accept(self)
         if meth.ret is not None:
             meth.ret.accept(self)
+        if meth.typeop is not None:
+            meth.typeop.accept(self)
+        if meth.T is not None:
+            meth.T.accept(self)
 
     def visitMethodDefn(self, meth):
         meth.decl.accept(self)
         self.visitBlock(meth)
 
+    def visitFunctionDecl(self, fun):
+        self.visitMethodDecl(fun)
+
+    def visitFunctionDefn(self, fd):
+        self.visitMethodDefn(fd)
+
     def visitConstructorDecl(self, ctor):
         self.visitMethodDecl(ctor)
 
@@ -384,6 +397,11 @@ class Typedef(Node):
         self.fromtype = fromtype
         self.totypename = totypename
 
+class Using(Node):
+    def __init__(self, type):
+        Node.__init__(self)
+        self.type = type
+
 class ForwardDecl(Node):
     def __init__(self, pqname, cls=0, struct=0):
         assert (not cls and struct) or (cls and not struct)
@@ -447,14 +465,18 @@ class FriendClassDecl(Node):
 
 class MethodDecl(Node):
     def __init__(self, name, params=[ ], ret=Type('void'),
-                 virtual=0, const=0, pure=0, static=0,
-                 typeop=None):
+                 virtual=0, const=0, pure=0, static=0, warn_unused=0,
+                 inline=0, force_inline=0,
+                 typeop=None,
+                 T=None):
         assert not (virtual and static)
         assert not pure or virtual      # pure => virtual
         assert not (static and typeop)
         assert not (name and typeop)
         assert name is None or isinstance(name, str)
         assert not isinstance(ret, list)
+        for decl in params:  assert not isinstance(decl, str)
+        assert not isinstance(T, int)
 
         if typeop is not None:
             ret = None
@@ -467,24 +489,50 @@ class MethodDecl(Node):
         self.const = const              # bool
         self.pure = pure                # bool
         self.static = static            # bool
+        self.warn_unused = warn_unused  # bool
+        self.force_inline = (force_inline or T) # bool
+        self.inline = inline            # bool
         self.typeop = typeop            # Type or None
+        self.T = T                      # Type or None
 
     def __deepcopy__(self, memo):
         return MethodDecl(
             self.name,
-            copy.deepcopy(self.params, memo),
-            copy.deepcopy(self.ret, memo),
-            self.virtual, self.const, self.pure, self.static,
-            copy.deepcopy(self.typeop, memo))
+            params=copy.deepcopy(self.params, memo),
+            ret=copy.deepcopy(self.ret, memo),
+            virtual=self.virtual,
+            const=self.const,
+            pure=self.pure,
+            static=self.static,
+            warn_unused=self.warn_unused,
+            inline=self.inline,
+            force_inline=self.force_inline,
+            typeop=copy.deepcopy(self.typeop, memo),
+            T=copy.deepcopy(self.T, memo))
 
 class MethodDefn(Block):
     def __init__(self, decl):
         Block.__init__(self)
         self.decl = decl
 
+class FunctionDecl(MethodDecl):
+    def __init__(self, name, params=[ ], ret=Type('void'),
+                 static=0, warn_unused=0,
+                 inline=0, force_inline=0,
+                 T=None):
+        MethodDecl.__init__(self, name, params=params, ret=ret,
+                            static=static, warn_unused=warn_unused,
+                            inline=inline, force_inline=force_inline,
+                            T=T)
+
+class FunctionDefn(MethodDefn):
+    def __init__(self, decl):
+        MethodDefn.__init__(self, decl)
+
 class ConstructorDecl(MethodDecl):
-    def __init__(self, name, params=[ ], explicit=0):
-        MethodDecl.__init__(self, name, params=params, ret=None)
+    def __init__(self, name, params=[ ], explicit=0, force_inline=0):
+        MethodDecl.__init__(self, name, params=params, ret=None,
+                            force_inline=force_inline)
         self.explicit = explicit
 
     def __deepcopy__(self, memo):
@@ -498,12 +546,16 @@ class ConstructorDefn(MethodDefn):
         self.memberinits = memberinits
 
 class DestructorDecl(MethodDecl):
-    def __init__(self, name, virtual=0):
+    def __init__(self, name, virtual=0, force_inline=0, inline=0):
         MethodDecl.__init__(self, name, params=[ ], ret=None,
-                            virtual=virtual)
+                            virtual=virtual,
+                            force_inline=force_inline, inline=inline)
 
     def __deepcopy__(self, memo):
-        return DestructorDecl(self.name, self.virtual)
+        return DestructorDecl(self.name,
+                              virtual=self.virtual,
+                              force_inline=self.force_inline,
+                              inline=self.inline)
 
         
 class DestructorDefn(MethodDefn):
@@ -617,7 +669,7 @@ class ExprCall(Node):
     def __init__(self, func, args=[ ]):
         assert hasattr(func, 'accept')
         assert isinstance(args, list)
-        for arg in args:  assert not isinstance(arg, str)
+        for arg in args:  assert arg and not isinstance(arg, str)
 
         Node.__init__(self)
         self.func = func
@@ -657,6 +709,7 @@ class StmtDecl(Node):
     def __init__(self, decl, init=None, initargs=None):
         assert not (init and initargs)
         assert not isinstance(init, str) # easy to confuse with Decl
+        assert not isinstance(init, list)
         assert not isinstance(decl, tuple)
         
         Node.__init__(self)
@@ -720,6 +773,7 @@ class StmtSwitch(Block):
         assert not isinstance(case, str)
         assert (isinstance(block, StmtBreak)
                 or isinstance(block, StmtReturn)
+                or isinstance(block, StmtSwitch)
                 or (hasattr(block, 'stmts')
                     and (isinstance(block.stmts[-1], StmtBreak)
                          or isinstance(block.stmts[-1], StmtReturn))))
@@ -727,12 +781,18 @@ class StmtSwitch(Block):
         self.addstmt(block)
         self.nr_cases += 1
 
+    def addfallthrough(self, case):
+        self.addstmt(case)
+        self.nr_cases += 1
+
 class StmtBreak(Node):
     def __init__(self):
         Node.__init__(self)
 
 class StmtExpr(Node):
     def __init__(self, expr):
+        assert expr is not None
+        
         Node.__init__(self)
         self.expr = expr
 
@@ -740,3 +800,6 @@ class StmtReturn(Node):
     def __init__(self, expr=None):
         Node.__init__(self)
         self.expr = expr
+
+StmtReturn.TRUE = StmtReturn(ExprLiteral.TRUE)
+StmtReturn.FALSE = StmtReturn(ExprLiteral.FALSE)
