@@ -71,9 +71,9 @@ static const uintN JS_GCTHING_ALIGN = 8;
 static const uintN JS_GCTHING_ZEROBITS = 3;
 
 /* Scalar typedefs. */
-typedef uint8  jsbytecode;
-typedef uint8  jssrcnote;
-typedef uint32 jsatomid;
+typedef uint8       jsbytecode;
+typedef uint8       jssrcnote;
+typedef uintptr_t   jsatomid;
 
 /* Struct typedefs. */
 typedef struct JSArgumentFormatMap  JSArgumentFormatMap;
@@ -92,13 +92,10 @@ typedef struct JSTreeContext        JSTreeContext;
 typedef struct JSTryNote            JSTryNote;
 
 /* Friend "Advanced API" typedefs. */
-typedef struct JSAtomList           JSAtomList;
-typedef struct JSAtomListElement    JSAtomListElement;
 typedef struct JSAtomMap            JSAtomMap;
 typedef struct JSAtomState          JSAtomState;
 typedef struct JSCodeSpec           JSCodeSpec;
 typedef struct JSPrinter            JSPrinter;
-typedef struct JSRegExpStatics      JSRegExpStatics;
 typedef struct JSStackHeader        JSStackHeader;
 typedef struct JSSubString          JSSubString;
 typedef struct JSNativeTraceInfo    JSNativeTraceInfo;
@@ -126,6 +123,7 @@ class JSFixedString;
 class JSStaticAtom;
 class JSRope;
 class JSAtom;
+struct JSDefinition;
 
 namespace js {
 
@@ -158,12 +156,11 @@ struct Token;
 struct TokenPos;
 struct TokenPtr;
 
-class ContextAllocPolicy;
-class SystemAllocPolicy;
+class TempAllocPolicy;
 
 template <class T,
           size_t MinInlineCapacity = 0,
-          class AllocPolicy = ContextAllocPolicy>
+          class AllocPolicy = TempAllocPolicy>
 class Vector;
 
 template <class>
@@ -172,14 +169,19 @@ struct DefaultHasher;
 template <class Key,
           class Value,
           class HashPolicy = DefaultHasher<Key>,
-          class AllocPolicy = ContextAllocPolicy>
+          class AllocPolicy = TempAllocPolicy>
 class HashMap;
 
 template <class T,
           class HashPolicy = DefaultHasher<T>,
-          class AllocPolicy = ContextAllocPolicy>
+          class AllocPolicy = TempAllocPolicy>
 class HashSet;
 
+template <typename K,
+          typename V,
+          size_t InlineElems>
+class InlineMap;
+
 class PropertyCache;
 struct PropertyCacheEntry;
 
@@ -187,6 +189,13 @@ struct Shape;
 struct EmptyShape;
 class Bindings;
 
+class MultiDeclRange;
+class ParseMapPool;
+class DefnOrHeader;
+typedef js::InlineMap<JSAtom *, JSDefinition *, 24> AtomDefnMap;
+typedef js::InlineMap<JSAtom *, jsatomid, 24> AtomIndexMap;
+typedef js::InlineMap<JSAtom *, DefnOrHeader, 24> AtomDOHMap;
+
 } /* namespace js */
 
 } /* export "C++" */
@@ -242,7 +251,7 @@ typedef void
                         void      *callerdata);
 
 typedef void
-(* JSSourceHandler)(const char *filename, uintN lineno, jschar *str,
+(* JSSourceHandler)(const char *filename, uintN lineno, const jschar *str,
                     size_t length, void **listenerTSData, void *closure);
 
 /*
