@@ -158,7 +158,8 @@ struct VectorImpl<T, N, AP, true>
     static inline bool growTo(Vector<T,N,AP> &v, size_t newcap) {
         JS_ASSERT(!v.usingInlineStorage());
         size_t bytes = sizeof(T) * newcap;
-        T *newbuf = reinterpret_cast<T *>(v.realloc_(v.mBegin, bytes));
+        size_t oldBytes = sizeof(T) * v.mCapacity;
+        T *newbuf = reinterpret_cast<T *>(v.realloc_(v.mBegin, oldBytes, bytes));
         if (!newbuf)
             return false;
         v.mBegin = newbuf;
@@ -181,7 +182,7 @@ struct VectorImpl<T, N, AP, true>
  * N requirements:
  *  - any value, however, N is clamped to min/max values
  * AllocPolicy:
- *  - see "Allocation policies" in jstl.h (default ContextAllocPolicy)
+ *  - see "Allocation policies" in jsalloc.h (default js::TempAllocPolicy)
  *
  * N.B: Vector is not reentrant: T member functions called during Vector member
  *      functions must not call back into the same object.
@@ -208,12 +209,30 @@ class Vector : private AllocPolicy
 
     /* compute constants */
 
+    /*
+     * Consider element size to be 1 for buffer sizing if there are
+     * 0 inline elements. This allows us to compile when the definition
+     * of the element type is not visible here.
+     *
+     * Explicit specialization is only allowed at namespace scope, so
+     * in order to keep everything here, we use a dummy template
+     * parameter with partial specialization.
+     */
+    template <int M, int Dummy>
+    struct ElemSize {
+        static const size_t result = sizeof(T);
+    };
+    template <int Dummy>
+    struct ElemSize<0, Dummy> {
+        static const size_t result = 1;
+    };
+
     static const size_t sInlineCapacity =
-        tl::Min<N, sMaxInlineBytes / sizeof(T)>::result;
+        tl::Min<N, sMaxInlineBytes / ElemSize<N, 0>::result>::result;
 
     /* Calculate inline buffer size; avoid 0-sized array. */
     static const size_t sInlineBytes =
-        tl::Max<1, sInlineCapacity * sizeof(T)>::result;
+        tl::Max<1, sInlineCapacity * ElemSize<N, 0>::result>::result;
 
     /* member data */
 
@@ -360,6 +379,9 @@ class Vector : private AllocPolicy
     /* Shorthand for shrinkBy(length()). */
     void clear();
 
+    /* Clears and releases any heap-allocated storage. */
+    void clearAndFree();
+
     /* Potentially fallible append operations. */
     bool append(const T &t);
     bool appendN(const T &t, size_t n);
@@ -635,6 +657,23 @@ Vector<T,N,AP>::clear()
     mLength = 0;
 }
 
+template <class T, size_t N, class AP>
+inline void
+Vector<T,N,AP>::clearAndFree()
+{
+    clear();
+
+    if (usingInlineStorage())
+        return;
+
+    this->free_(beginNoCheck());
+    mBegin = (T *)storage.addr();
+    mCapacity = sInlineCapacity;
+#ifdef DEBUG
+    mReserved = 0;
+#endif
+}
+
 template <class T, size_t N, class AP>
 JS_ALWAYS_INLINE bool
 Vector<T,N,AP>::append(const T &t)
