@@ -331,7 +331,10 @@ mjit::Compiler::jsop_bitop(JSOp op)
 {
     FrameEntry *rhs = frame.peek(-1);
     FrameEntry *lhs = frame.peek(-2);
-
+    
+    if (tryBinaryConstantFold(cx, frame, op, lhs, rhs))
+        return;
+    
     VoidStub stub;
     switch (op) {
       case JSOP_BITOR:
@@ -353,14 +356,26 @@ mjit::Compiler::jsop_bitop(JSOp op)
         JS_NOT_REACHED("wat");
         return;
     }
+    
+    int32_t rhsInt = 0;
+    int32_t lhsInt = 0;
+    bool rhsConstant = false;
+    bool lhsConstant = false;
+    
+    if (rhs->isConstant() && rhs->getValue().isPrimitive()) {
+        ValueToECMAInt32(cx, rhs->getValue(), &rhsInt);
+        rhsConstant = true;
+    }
+    if (lhs->isConstant() && lhs->getValue().isPrimitive()) {
+        ValueToECMAInt32(cx, lhs->getValue(), &lhsInt);
+        lhsConstant = true;
+    }    
 
     bool lhsIntOrDouble = !(lhs->isNotType(JSVAL_TYPE_DOUBLE) && 
                             lhs->isNotType(JSVAL_TYPE_INT32));
     
     /* Fast-path double to int conversion. */
-    if (!lhs->isConstant() && rhs->isConstant() && lhsIntOrDouble &&
-        rhs->isType(JSVAL_TYPE_INT32) && rhs->getValue().toInt32() == 0 &&
-        (op == JSOP_BITOR || op == JSOP_LSH)) {
+    if (rhsConstant && rhsInt == 0 && lhsIntOrDouble  && (op == JSOP_BITOR || op == JSOP_LSH)) {
         RegisterID reg = frame.copyDataIntoReg(lhs);
         if (lhs->isType(JSVAL_TYPE_INT32)) {
             frame.popn(2);
@@ -390,8 +405,9 @@ mjit::Compiler::jsop_bitop(JSOp op)
     }
 
     /* We only want to handle integers here. */
-    if (rhs->isNotType(JSVAL_TYPE_INT32) || lhs->isNotType(JSVAL_TYPE_INT32) || 
-        (op == JSOP_URSH && rhs->isConstant() && rhs->getValue().toInt32() % 32 == 0)) {
+    if ((!rhsConstant && rhs->isNotType(JSVAL_TYPE_INT32)) || 
+        (!lhsConstant && lhs->isNotType(JSVAL_TYPE_INT32)) || 
+        (op == JSOP_URSH && rhsConstant && rhsInt % 32 == 0)) {
         prepareStubCall(Uses(2));
         INLINE_STUBCALL(stub);
         frame.popn(2);
@@ -416,38 +432,6 @@ mjit::Compiler::jsop_bitop(JSOp op)
         stubNeeded = true;
     }
 
-    if (lhs->isConstant() && rhs->isConstant()) {
-        int32 L = lhs->getValue().toInt32();
-        int32 R = rhs->getValue().toInt32();
-
-        frame.popn(2);
-        switch (op) {
-          case JSOP_BITOR:
-            frame.push(Int32Value(L | R));
-            return;
-          case JSOP_BITXOR:
-            frame.push(Int32Value(L ^ R));
-            return;
-          case JSOP_BITAND:
-            frame.push(Int32Value(L & R));
-            return;
-          case JSOP_LSH:
-            frame.push(Int32Value(L << R));
-            return;
-          case JSOP_URSH: 
-          {
-            uint32 unsignedL;
-            if (ValueToECMAUint32(cx, lhs->getValue(), (uint32_t*)&unsignedL)) {
-                frame.push(NumberValue(uint32(unsignedL >> (R & 31))));
-                return;
-            }
-            break;
-          }
-          default:
-            JS_NOT_REACHED("say wat");
-        }
-    }
-
     RegisterID reg;
 
     switch (op) {
@@ -456,21 +440,24 @@ mjit::Compiler::jsop_bitop(JSOp op)
       case JSOP_BITAND:
       {
         /* Commutative, and we're guaranteed both are ints. */
-        if (lhs->isConstant()) {
+        if (lhsConstant) {
             JS_ASSERT(!rhs->isConstant());
             FrameEntry *temp = rhs;
             rhs = lhs;
             lhs = temp;
+            
+            rhsInt = lhsInt;
+            rhsConstant = true;
         }
 
         reg = frame.ownRegForData(lhs);
-        if (rhs->isConstant()) {
+        if (rhsConstant) {
             if (op == JSOP_BITAND)
-                masm.and32(Imm32(rhs->getValue().toInt32()), reg);
+                masm.and32(Imm32(rhsInt), reg);
             else if (op == JSOP_BITXOR)
-                masm.xor32(Imm32(rhs->getValue().toInt32()), reg);
+                masm.xor32(Imm32(rhsInt), reg);
             else
-                masm.or32(Imm32(rhs->getValue().toInt32()), reg);
+                masm.or32(Imm32(rhsInt), reg);
         } else if (frame.shouldAvoidDataRemat(rhs)) {
             if (op == JSOP_BITAND)
                 masm.and32(masm.payloadOf(frame.addressOf(rhs)), reg);
@@ -495,9 +482,9 @@ mjit::Compiler::jsop_bitop(JSOp op)
       case JSOP_URSH:
       {
         /* Not commutative. */
-        if (rhs->isConstant()) {
+        if (rhsConstant) {
             RegisterID reg = frame.ownRegForData(lhs);
-            int shift = rhs->getValue().toInt32() & 0x1F;
+            int shift = rhsInt & 0x1F;
 
             if (shift) {
                 if (op == JSOP_LSH)
@@ -536,9 +523,9 @@ mjit::Compiler::jsop_bitop(JSOp op)
                 masm.move(rr, reg);
         } else {
             frame.pinReg(rr);
-            if (lhs->isConstant()) {
+            if (lhsConstant) {
                 reg = frame.allocReg();
-                masm.move(Imm32(lhs->getValue().toInt32()), reg);
+                masm.move(Imm32(lhsInt), reg);
             } else {
                 reg = frame.copyDataIntoReg(lhs);
             }
@@ -567,9 +554,8 @@ mjit::Compiler::jsop_bitop(JSOp op)
         OOL_STUBCALL(stub);
     }
 
-    frame.pop();
-    frame.pop();
-
+    frame.popn(2);
+    
     if (op == JSOP_URSH)
         frame.pushNumber(reg, true);
     else
