@@ -20,7 +20,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,29 +37,21 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-var BUGNUMBER = 354145;
-var summary = 'Immutable XML';
+var summary = "Do not crash during gc()";
+var BUGNUMBER = 327691;
 var actual = 'No Crash';
 var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
+var A=<y/>;
+var B=A.h;
+var D=B.h;
+B.h=D.h;
+B[0]=B.h;
 
-var list = <></>
-var N = 10;
-for (var i = 0; i != N; ++i)
-  list[i] = <{"a"+i}/>;
-
-function prepare()
-{
-  for (var i = N - 1; i >= 0; --i)
-    delete list[i];
-  gc();
-  return "test";
-}
-
-list.child({ toString: prepare });
+gc();
 
 TEST(1, expect, actual);
 
