@@ -41,39 +41,45 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-START("13.5.4.14 - XMLList length()");
+START("13.5.4.4 - XMLList child()");
 
-TEST(1, true, XMLList.prototype.hasOwnProperty("length"));
+TEST(1, true, XMLList.prototype.hasOwnProperty("child"));
 
-x = <><alpha>one</alpha></>;
+// Test with XMLList of size 0
+x = new XMLList()
+TEST(2, "xml", typeof(x.child("bravo")));
+TEST_XML(3, "", x.child("bravo"));
 
-TEST(2, 1, x.length());
+// Test with XMLList of size 1
+x += <alpha>one<bravo>two</bravo></alpha>;   
+TEST(4, "xml", typeof(x.child("bravo")));
+TEST_XML(5, "<bravo>two</bravo>", x.child("bravo"));
 
-x = <><alpha>one</alpha><bravo>two</bravo></>;
+x += <charlie><bravo>three</bravo></charlie>;
+TEST(6, "xml", typeof(x.child("bravo")));
 
-TEST(2, 2, x.length());
+correct = <><bravo>two</bravo><bravo>three</bravo></>;
+TEST(7, correct, x.child("bravo"));
 
-emps =
-<employees>
-    <employee>
-        <name>John</name>
-    </employee>
-    <employee>
-        <name>Sue</name>
-    </employee>
-</employees>   
+// Test no match, null and undefined
+TEST(8, "xml", typeof(x.child("foobar")));
+TEST_XML(9, "", x.child("foobar"));
 
-correct =
-<employees>
-    <employee>
-        <prefix>Mr.</prefix>
-        <name>John</name>
-    </employee>
-    <employee>
-        <name>Sue</name>
-    </employee>
-</employees>   
+try {
+  x.child(null);
+  SHOULD_THROW(10);
+} catch (ex) {
+  TEST(10, "TypeError", ex.name);
+}
 
-TEST(3,2,emps..name.length());
+// Test numeric inputs
+x =
+<alpha>
+    <bravo>one</bravo>
+    <charlie>two</charlie>
+</alpha>;
+
+TEST(12, <bravo>one</bravo>, x.child(0));
+TEST(13, <charlie>two</charlie>, x.child(1));
 
 END();
