@@ -37,25 +37,55 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-var BUGNUMBER = 354151;
-var summary = 'Bad assumptions about Array elements';
+var BUGNUMBER = 352846;
+var summary = 'Passing unrooted value to OBJ_DEFAULT_VALUE is GC hazard';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-function getter() { return 1; }
-function setter(v) { return v; }
+var counter = 0;
 
-var xml = <a xmlns="foo:bar"/>;
+function prepare_xml()
+{
+  delete XML.prototype.function::toString;
+  Object.defineProperty(Object.prototype, "toString",
+                        { get: toSource_getter, enumerable: true,
+                          configurable: true });
+  return new XML("<a>xml_text</a>");
+}
 
-Array.prototype.__defineGetter__(0, getter);
-Array.prototype.__defineSetter__(0, setter);
+function toSource_getter()
+{
+  // Make sure that lastInternalResult does not contain prepare_xml
+  var tmp = { toSource: function() { return [1,2,3]; } };
+  uneval(tmp);
 
-xml.namespace();
+  if (counter++ < 2) return undefined;
 
-delete Array.prototype[0];
+  // run GC onr overwrite the heap
+  if (typeof gc == "function") gc();
+  var x = 1e-100;
+  for (var i = 0; i != 50000; ++i)
+    var x2 = x / 4;
+
+  // Return a function that would access this in non-trivial way to
+  // check if prepare_xml() was rooted.
+  return function() {
+    print("xxx");
+    return this.toXMLString();
+  };
+}
+
+var obj = { };
+Object.defineProperty(obj, "length",
+{
+  get: prepare_xml,
+  enumerable: true,
+  configurable: true
+});
+Array.reverse(obj);
 
 TEST(1, expect, actual);
 
