@@ -38,7 +38,7 @@
 
 
 /**
-   File Name:          10.2.3-2.js
+   File Name:          10.2.3-1.js
    ECMA Section:       10.2.3 Function and Anonymous Code
    Description:
 
@@ -54,37 +54,31 @@
    Date:               12 november 1997
 */
 
-var SECTION = "10.2.3-2";
+var SECTION = "10.2.3-1";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Function and Anonymous Code";
+var TITLE   = "Eval Code";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-var o = new MyObject("hello");
+var o = new MyObject("hello")
 
-new TestCase( SECTION,
-	      "MyFunction(\"PASSED!\")",
-	      "PASSED!",
-	      MyFunction("PASSED!") );
+  new TestCase( SECTION,
+		"var o = new MyObject('hello'); o.THIS == x",
+		true,
+		o.THIS == o );
 
 var o = MyFunction();
 
 new TestCase( SECTION,
-	      "MyOtherFunction(true);",
-	      false,
-	      MyOtherFunction(true) );
+	      "var o = MyFunction(); o == this",
+	      true,
+	      o == this );
 
 test();
 
 function MyFunction( value ) {
-  var x = value;
-  delete x;
-  return x;
-}
-function MyOtherFunction(value) {
-  var x = value;
-  return delete x;
+  return this;
 }
 function MyObject( value ) {
   this.THIS = this;
