@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Bertrand Le Roy
+ * Contributor(s): moz_bug_r_a4
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,43 +36,25 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 350256;
-var summary = 'Array.apply maximum arguments: 2^19-1024';
+var BUGNUMBER = 315509;
+var summary = 'Array.prototype.unshift on Arrays with holes';
 var actual = '';
 var expect = '';
 
-expectExitCode(0);
-expectExitCode(5);
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-//-----------------------------------------------------------------------------
-test(Math.pow(2, 19)-1024);
-//-----------------------------------------------------------------------------
+var a = [0,1,2,3,4];
+delete a[1];
 
-function test(length)
-{
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
+expect = '0,,2,3,4';
+actual = a.toString();
 
-  var a = new Array();
-  a[length - 2] = 'length-2';
-  a[length - 1] = 'length-1';
+reportCompare(expect, actual, summary);
 
-  var b = Array.apply(null, a);
+a.unshift('a','b');
 
-  expect = length + ',length-2,length-1';
-  actual = b.length + "," + b[length - 2] + "," + b[length - 1];
-  reportCompare(expect, actual, summary);
-
-  function f() {
-    return arguments.length + "," + arguments[length - 2] + "," +
-      arguments[length - 1];
-  }
-
-  expect = length + ',length-2,length-1';
-  actual = f.apply(null, a);
-
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
-}
+expect = 'a,b,0,,2,3,4';
+actual = a.toString();
+ 
+reportCompare(expect, actual, summary);
