@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,8 +36,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 387951;
-var summary = 'Do not assert: cg->stackDepth >= 0';
+var BUGNUMBER = 349482;
+var summary = 'Decompiling try/catch in with() should not crash';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
@@ -52,7 +52,8 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  "" + (function() { if(delete(null?0:{})){[]} });
+  var f = function() { with({})     { try{}catch(e){} } }
+  print(f.toString());
 
   reportCompare(expect, actual, summary);
 
