@@ -46,7 +46,6 @@
 #include "nsIXPConnect.h"
 #include "nsIFile.h"
 #include "nsAutoPtr.h"
-#include "nsIFastLoadService.h"
 #include "nsIObjectInputStream.h"
 #include "nsIObjectOutputStream.h"
 #include "nsITimer.h"
@@ -68,28 +67,6 @@ using namespace mozilla::scache;
     { 0xbb, 0xef, 0xf0, 0xcc, 0xb5, 0xfa, 0x64, 0xb6 }}
 #define MOZJSCOMPONENTLOADER_CONTRACTID "@mozilla.org/moz/jsloader;1"
 
-// nsIFastLoadFileIO implementation for component fastload
-class nsXPCFastLoadIO : public nsIFastLoadFileIO
-{
- public:
-    NS_DECL_ISUPPORTS
-    NS_DECL_NSIFASTLOADFILEIO
-
-    nsXPCFastLoadIO(nsIFile *file) : mFile(file), mTruncateOutputFile(true) {}
-
-    void SetInputStream(nsIInputStream *stream) { mInputStream = stream; }
-    void SetOutputStream(nsIOutputStream *stream) { mOutputStream = stream; }
-
- private:
-    ~nsXPCFastLoadIO() {}
-
-    nsCOMPtr<nsIFile> mFile;
-    nsCOMPtr<nsIInputStream> mInputStream;
-    nsCOMPtr<nsIOutputStream> mOutputStream;
-    bool mTruncateOutputFile;
-};
-
-
 class mozJSComponentLoader : public mozilla::ModuleLoader,
                              public xpcIJSModuleLoader,
                              public nsIObserver
