@@ -1,4 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=4 sw=4 et tw=79:
  *
  * ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
@@ -44,22 +45,40 @@
 #define xpcinlines_h___
 
 /***************************************************************************/
+PRBool
+xpc::PtrAndPrincipalHashKey::KeyEquals(const PtrAndPrincipalHashKey* aKey) const
+{
+    if(aKey->mPtr != mPtr)
+        return PR_FALSE;
+    if(aKey->mPrincipal == mPrincipal)
+        return PR_TRUE;
+
+    PRBool equals;
+    if(NS_FAILED(mPrincipal->EqualsIgnoringDomain(aKey->mPrincipal, &equals)))
+    {
+        NS_ERROR("we failed, guessing!");
+        return PR_FALSE;
+    }
+
+    return equals;
+}
+
 inline void
 XPCJSRuntime::AddVariantRoot(XPCTraceableVariant* variant)
 {
-    variant->AddToRootSet(GetJSRuntime(), &mVariantRoots);
+    variant->AddToRootSet(GetMapLock(), &mVariantRoots);
 }
 
 inline void
 XPCJSRuntime::AddWrappedJSRoot(nsXPCWrappedJS* wrappedJS)
 {
-    wrappedJS->AddToRootSet(GetJSRuntime(), &mWrappedJSRoots);
+    wrappedJS->AddToRootSet(GetMapLock(), &mWrappedJSRoots);
 }
 
 inline void
 XPCJSRuntime::AddObjectHolderRoot(XPCJSObjectHolder* holder)
 {
-    holder->AddToRootSet(GetJSRuntime(), &mObjectHolderRoots);
+    holder->AddToRootSet(GetMapLock(), &mObjectHolderRoots);
 }
 
 /***************************************************************************/
@@ -144,17 +163,19 @@ XPCCallContext::GetPrevCallContext() const
 }
 
 inline JSObject*
-XPCCallContext::GetOperandJSObject() const
+XPCCallContext::GetScopeForNewJSObjects() const
 {
-    CHECK_STATE(HAVE_OBJECT);
-    return mOperandJSObject;
+    CHECK_STATE(HAVE_SCOPE);
+    return mScopeForNewJSObjects;
 }
 
-inline JSObject*
-XPCCallContext::GetCurrentJSObject() const
+inline void
+XPCCallContext::SetScopeForNewJSObjects(JSObject *scope)
 {
-    CHECK_STATE(HAVE_OBJECT);
-    return mCurrentJSObject;
+    NS_ABORT_IF_FALSE(mState == HAVE_CONTEXT, "wrong call context state");
+    NS_ABORT_IF_FALSE(scope->compartment() == mJSContext->compartment, "wrong compartment");
+    mScopeForNewJSObjects = scope;
+    mState = HAVE_SCOPE;
 }
 
 inline JSObject*
@@ -168,7 +189,11 @@ inline nsISupports*
 XPCCallContext::GetIdentityObject() const
 {
     CHECK_STATE(HAVE_OBJECT);
-    return mWrapper->GetIdentityObject();
+    if(mWrapper)
+        return mWrapper->GetIdentityObject();
+    return mFlattenedJSObject ?
+           static_cast<nsISupports*>(xpc_GetJSPrivate(mFlattenedJSObject)) :
+           nsnull;
 }
 
 inline XPCWrappedNative*
@@ -181,6 +206,15 @@ XPCCallContext::GetWrapper() const
     return mWrapper;
 }
 
+inline XPCWrappedNativeProto*
+XPCCallContext::GetProto() const
+{
+    CHECK_STATE(HAVE_OBJECT);
+    if(mWrapper)
+        return mWrapper->GetProto();
+    return mFlattenedJSObject ? GetSlimWrapperProto(mFlattenedJSObject) : nsnull;
+}
+
 inline JSBool
 XPCCallContext::CanGetTearOff() const
 {
@@ -237,14 +271,10 @@ XPCCallContext::GetMember() const
 inline JSBool
 XPCCallContext::HasInterfaceAndMember() const
 {
-    return mState >= HAVE_NAME && mInterface && (mMember
-#ifdef XPC_IDISPATCH_SUPPORT
-        || mIDispatchMember
-#endif
-        );
+    return mState >= HAVE_NAME && mInterface && mMember;
 }
 
-inline jsval
+inline jsid
 XPCCallContext::GetName() const
 {
     CHECK_STATE(HAVE_NAME);
@@ -294,15 +324,15 @@ XPCCallContext::SetRetVal(jsval val)
         *mRetVal = val;
 }
 
-inline jsval
+inline jsid
 XPCCallContext::GetResolveName() const
 {
     CHECK_STATE(HAVE_CONTEXT);
     return mThreadData->GetResolveName();
 }
 
-inline jsval
-XPCCallContext::SetResolveName(jsval name)
+inline jsid
+XPCCallContext::SetResolveName(jsid name)
 {
     CHECK_STATE(HAVE_CONTEXT);
     return mThreadData->SetResolveName(name);
@@ -322,22 +352,6 @@ XPCCallContext::SetResolvingWrapper(XPCWrappedNative* w)
     return mThreadData->SetResolvingWrapper(w);
 }
 
-inline JSObject*
-XPCCallContext::GetCallee() const
-{
-    NS_ASSERTION(mCallerLanguage == NATIVE_CALLER,
-                 "GetCallee() doesn't make sense");
-    return mCallee;
-}
-
-inline void
-XPCCallContext::SetCallee(JSObject* callee)
-{
-    NS_ASSERTION(mCallerLanguage == NATIVE_CALLER,
-                 "SetCallee() doesn't make sense");
-    mCallee = callee;
-}
-
 inline PRUint16
 XPCCallContext::GetMethodIndex() const
 {
@@ -383,7 +397,7 @@ XPCNativeInterface::GetNameString() const
 }
 
 inline XPCNativeMember*
-XPCNativeInterface::FindMember(jsval name) const
+XPCNativeInterface::FindMember(jsid name) const
 {
     const XPCNativeMember* member = mMembers;
     for(int i = (int) mMemberCount; i > 0; i--, member++)
@@ -400,18 +414,10 @@ XPCNativeInterface::HasAncestor(const nsIID* iid) const
     return found;
 }
 
-inline void
-XPCNativeInterface::DealWithDyingGCThings(JSContext* cx, XPCJSRuntime* rt)
-{
-    XPCNativeMember* member = mMembers;
-    for(int i = (int) mMemberCount; i > 0; i--, member++)
-        member->DealWithDyingGCThings(cx, rt);
-}
-
 /***************************************************************************/
 
 inline JSBool
-XPCNativeSet::FindMember(jsval name, XPCNativeMember** pMember,
+XPCNativeSet::FindMember(jsid name, XPCNativeMember** pMember,
                          PRUint16* pInterfaceIndex) const
 {
     XPCNativeInterface* const * iface;
@@ -449,7 +455,7 @@ XPCNativeSet::FindMember(jsval name, XPCNativeMember** pMember,
 }
 
 inline JSBool
-XPCNativeSet::FindMember(jsval name, XPCNativeMember** pMember,
+XPCNativeSet::FindMember(jsid name, XPCNativeMember** pMember,
                          XPCNativeInterface** pInterface) const
 {
     PRUint16 index;
@@ -460,7 +466,7 @@ XPCNativeSet::FindMember(jsval name, XPCNativeMember** pMember,
 }
 
 inline JSBool
-XPCNativeSet::FindMember(jsval name,
+XPCNativeSet::FindMember(jsid name,
                          XPCNativeMember** pMember,
                          XPCNativeInterface** pInterface,
                          XPCNativeSet* protoSet,
@@ -488,7 +494,7 @@ XPCNativeSet::FindMember(jsval name,
 }
 
 inline XPCNativeInterface*
-XPCNativeSet::FindNamedInterface(jsval name) const
+XPCNativeSet::FindNamedInterface(jsid name) const
 {
     XPCNativeInterface* const * pp = mInterfaces;
 
@@ -533,8 +539,12 @@ XPCNativeSet::HasInterface(XPCNativeInterface* aInterface) const
 inline JSBool
 XPCNativeSet::HasInterfaceWithAncestor(XPCNativeInterface* aInterface) const
 {
-    const nsIID* iid = aInterface->GetIID();
+    return HasInterfaceWithAncestor(aInterface->GetIID());
+}
 
+inline JSBool
+XPCNativeSet::HasInterfaceWithAncestor(const nsIID* iid) const
+{
     // We can safely skip the first interface which is *always* nsISupports.
     XPCNativeInterface* const * pp = mInterfaces+1;
     for(int i = (int) mInterfaceCount; i > 1; i--, pp++)
@@ -598,75 +608,23 @@ inline void XPCNativeSet::ASSERT_NotMarked()
 inline
 JSObject* XPCWrappedNativeTearOff::GetJSObject() const
 {
-#ifdef XPC_IDISPATCH_SUPPORT
-    if(IsIDispatch())
-    {
-        XPCDispInterface * iface = GetIDispatchInfo();
-        return iface ? iface->GetJSObject() : nsnull;
-    }
-#endif
     return mJSObject;
 }
 
 inline
 void XPCWrappedNativeTearOff::SetJSObject(JSObject*  JSObj)
 {
-#ifdef XPC_IDISPATCH_SUPPORT
-    if(IsIDispatch())
-    {
-        XPCDispInterface* iface = GetIDispatchInfo();
-        if(iface)
-            iface->SetJSObject(JSObj);
-    }
-    else
-#endif
         mJSObject = JSObj;
 }
 
-#ifdef XPC_IDISPATCH_SUPPORT
-inline void
-XPCWrappedNativeTearOff::SetIDispatch(JSContext* cx)
-{
-    mJSObject = (JSObject*)(((jsword)
-        ::XPCDispInterface::NewInstance(cx,
-                                          mNative)) | 2);
-}
-
-inline XPCDispInterface* 
-XPCWrappedNativeTearOff::GetIDispatchInfo() const
-{
-    NS_ASSERTION((jsword)mJSObject & 2, "XPCWrappedNativeTearOff::GetIDispatchInfo "
-                                "called on a non IDispatch interface");
-    return reinterpret_cast<XPCDispInterface*>
-                           ((((jsword)mJSObject) & ~JSOBJECT_MASK));
-}
-
-inline JSBool
-XPCWrappedNativeTearOff::IsIDispatch() const
-{
-    return (JSBool)(((jsword)mJSObject) & IDISPATCH_BIT);
-}
-
-#endif
-
 inline
 XPCWrappedNativeTearOff::~XPCWrappedNativeTearOff()
 {
     NS_ASSERTION(!(GetInterface()||GetNative()||GetJSObject()), "tearoff not empty in dtor");
-#ifdef XPC_IDISPATCH_SUPPORT
-    if(IsIDispatch())
-        delete GetIDispatchInfo();
-#endif
 }
 
 /***************************************************************************/
 
-inline JSBool
-XPCWrappedNative::HasInterfaceNoQI(XPCNativeInterface* aInterface)
-{
-    return GetSet()->HasInterface(aInterface);
-}
-
 inline JSBool
 XPCWrappedNative::HasInterfaceNoQI(const nsIID& iid)
 {
@@ -706,13 +664,11 @@ XPCWrappedNative::SweepTearOffs()
 /***************************************************************************/
 
 inline JSBool
-xpc_ForcePropertyResolve(JSContext* cx, JSObject* obj, jsval idval)
+xpc_ForcePropertyResolve(JSContext* cx, JSObject* obj, jsid id)
 {
     jsval prop;
-    jsid id;
 
-    if(!JS_ValueToId(cx, idval, &id) ||
-       !JS_LookupPropertyById(cx, obj, id, &prop))
+    if(!JS_LookupPropertyById(cx, obj, id, &prop))
         return JS_FALSE;
     return JS_TRUE;
 }
@@ -721,8 +677,17 @@ inline JSObject*
 xpc_NewSystemInheritingJSObject(JSContext *cx, JSClass *clasp, JSObject *proto,
                                 JSObject *parent)
 {
-    return JS_NewSystemObject(cx, clasp, proto, parent,
-                              JS_IsSystemObject(cx, parent));
+    JSObject *obj;
+    if (clasp->flags & JSCLASS_IS_GLOBAL) {
+        obj = JS_NewGlobalObject(cx, clasp);
+        if (obj && proto)
+            JS_SetPrototype(cx, obj, proto);
+    } else {
+        obj = JS_NewObject(cx, clasp, proto, parent);
+    }
+    if (obj && JS_IsSystemObject(cx, parent) && !JS_MakeSystemObject(cx, obj))
+        obj = NULL;
+    return obj;
 }
 
 inline JSBool
@@ -746,11 +711,17 @@ xpc_SameScope(XPCWrappedNativeScope *objectscope, XPCWrappedNativeScope *xpcscop
     return JS_FALSE;
 }
 
+inline jsid
+GetRTIdByIndex(JSContext *cx, uintN index)
+{
+  XPCJSRuntime *rt = nsXPConnect::FastGetXPConnect()->GetRuntime();
+  return rt->GetStringID(index);
+}
+
 inline jsval
 GetRTStringByIndex(JSContext *cx, uintN index)
 {
-  XPCJSRuntime *rt = nsXPConnect::GetRuntimeInstance();
-  return ID_TO_VALUE(rt->GetStringID(index));
+  return STRING_TO_JSVAL(JSID_TO_STRING(GetRTIdByIndex(cx, index)));
 }
 
 inline
@@ -767,6 +738,27 @@ void ThrowBadResult(nsresult result, XPCCallContext& ccx)
                                result, ccx);
 }
 
+inline void
+XPCLazyCallContext::SetWrapper(XPCWrappedNative* wrapper,
+                               XPCWrappedNativeTearOff* tearoff)
+{
+    mWrapper = wrapper;
+    mTearOff = tearoff;
+    if(mTearOff)
+        mFlattenedJSObject = mTearOff->GetJSObject();
+    else
+        mFlattenedJSObject = mWrapper->GetFlatJSObject();
+}
+inline void
+XPCLazyCallContext::SetWrapper(JSObject* flattenedJSObject)
+{
+    NS_ASSERTION(IS_SLIM_WRAPPER_OBJECT(flattenedJSObject),
+                 "What kind of object is this?");
+    mWrapper = nsnull;
+    mTearOff = nsnull;
+    mFlattenedJSObject = flattenedJSObject;
+}
+
 /***************************************************************************/
 
 #endif /* xpcinlines_h___ */
