@@ -38,14 +38,12 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "jsapi.h"
-#include "jsstr.h"
 #include "jscntxt.h"  /* for error messages */
 #include "nsCOMPtr.h"
 #include "xpcprivate.h"
 #include "xpcinlines.h"
 #include "xpcquickstubs.h"
 #include "XPCWrapper.h"
-#include "XPCNativeWrapper.h"
 
 static inline QITableEntry *
 GetOffsets(nsISupports *identity, XPCWrappedNativeProto* proto)
@@ -119,6 +117,26 @@ LookupInterfaceOrAncestor(PRUint32 tableSize, const xpc_qsHashEntry *table,
     return entry;
 }
 
+// Apply |op| to |obj|, |id|, and |vp|. If |op| is a setter, treat the assignment as lenient.
+template<typename Op>
+static inline JSBool ApplyPropertyOp(JSContext *cx, Op op, JSObject *obj, jsid id, jsval *vp);
+
+template<>
+inline JSBool
+ApplyPropertyOp<JSPropertyOp>(JSContext *cx, JSPropertyOp op, JSObject *obj, jsid id, jsval *vp)
+{
+    return op(cx, obj, id, vp);
+}
+
+template<>
+inline JSBool
+ApplyPropertyOp<JSStrictPropertyOp>(JSContext *cx, JSStrictPropertyOp op, JSObject *obj,
+                                    jsid id, jsval *vp)
+{
+    return op(cx, obj, id, true, vp);
+}
+
+template<typename Op>
 static JSBool
 PropertyOpForwarder(JSContext *cx, uintN argc, jsval *vp)
 {
@@ -129,19 +147,24 @@ PropertyOpForwarder(JSContext *cx, uintN argc, jsval *vp)
 
     JSObject *callee = JSVAL_TO_OBJECT(JS_CALLEE(cx, vp));
     JSObject *obj = JS_THIS_OBJECT(cx, vp);
+    if (!obj)
+        return JS_FALSE;
     jsval v;
 
     if(!JS_GetReservedSlot(cx, callee, 0, &v))
         return JS_FALSE;
     JSObject *ptrobj = JSVAL_TO_OBJECT(v);
-    JSPropertyOp *popp = static_cast<JSPropertyOp *>(JS_GetPrivate(cx, ptrobj));
+    Op *popp = static_cast<Op *>(JS_GetPrivate(cx, ptrobj));
 
     if(!JS_GetReservedSlot(cx, callee, 1, &v))
         return JS_FALSE;
 
     jsval argval = (argc > 0) ? JS_ARGV(cx, vp)[0] : JSVAL_VOID;
+    jsid id;
+    if (!JS_ValueToId(cx, argval, &id))
+        return JS_FALSE;
     JS_SET_RVAL(cx, vp, argval);
-    return (*popp)(cx, obj, v, vp);
+    return ApplyPropertyOp<Op>(cx, *popp, obj, id, vp);
 }
 
 static void
@@ -154,57 +177,56 @@ PointerFinalize(JSContext *cx, JSObject *obj)
 static JSClass
 PointerHolderClass = {
     "Pointer", JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, PointerFinalize,
     JSCLASS_NO_OPTIONAL_MEMBERS
 };
 
+template<typename Op>
 static JSObject *
-GeneratePropertyOp(JSContext *cx, JSObject *obj, jsval idval, uintN argc,
-                   const char *name, JSPropertyOp pop)
+GeneratePropertyOp(JSContext *cx, JSObject *obj, jsid id, uintN argc, Op pop)
 {
     // The JS engine provides two reserved slots on function objects for
     // XPConnect to use. Use them to stick the necessary info here.
     JSFunction *fun =
-        JS_NewFunction(cx, reinterpret_cast<JSNative>(PropertyOpForwarder),
-                       argc, JSFUN_FAST_NATIVE, obj, name);
+        JS_NewFunctionById(cx, PropertyOpForwarder<Op>, argc, 0, obj, id);
     if(!fun)
         return JS_FALSE;
 
     JSObject *funobj = JS_GetFunctionObject(fun);
 
-    JSAutoTempValueRooter tvr(cx, OBJECT_TO_JSVAL(funobj));
+    js::AutoObjectRooter tvr(cx, funobj);
 
-    // Unfortunately, we cannot guarantee that JSPropertyOp is aligned. Use a
+    // Unfortunately, we cannot guarantee that Op is aligned. Use a
     // second object to work around this.
     JSObject *ptrobj = JS_NewObject(cx, &PointerHolderClass, nsnull, funobj);
     if(!ptrobj)
         return JS_FALSE;
-    JSPropertyOp *popp = new JSPropertyOp;
+    Op *popp = new Op;
     if(!popp)
         return JS_FALSE;
     *popp = pop;
     JS_SetPrivate(cx, ptrobj, popp);
 
     JS_SetReservedSlot(cx, funobj, 0, OBJECT_TO_JSVAL(ptrobj));
-    JS_SetReservedSlot(cx, funobj, 1, idval);
+    JS_SetReservedSlot(cx, funobj, 1, js::IdToJsval(id));
     return funobj;
 }
 
 static JSBool
-ReifyPropertyOps(JSContext *cx, JSObject *obj, jsval idval, jsid interned_id,
-                 const char *name, JSPropertyOp getter, JSPropertyOp setter,
+ReifyPropertyOps(JSContext *cx, JSObject *obj, jsid id, uintN orig_attrs,
+                 JSPropertyOp getter, JSStrictPropertyOp setter,
                  JSObject **getterobjp, JSObject **setterobjp)
 {
     // Generate both getter and setter and stash them in the prototype.
     jsval roots[2] = { JSVAL_NULL, JSVAL_NULL };
-    JSAutoTempValueRooter tvr(cx, 2, roots);
+    js::AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(roots), roots);
 
-    uintN attrs = JSPROP_SHARED;
+    uintN attrs = JSPROP_SHARED | (orig_attrs & JSPROP_ENUMERATE);
     JSObject *getterobj;
     if(getter)
     {
-        getterobj = GeneratePropertyOp(cx, obj, idval, 0, name, getter);
+        getterobj = GeneratePropertyOp(cx, obj, id, 0, getter);
         if(!getterobj)
             return JS_FALSE;
         roots[0] = OBJECT_TO_JSVAL(getterobj);
@@ -216,7 +238,7 @@ ReifyPropertyOps(JSContext *cx, JSObject *obj, jsval idval, jsid interned_id,
     JSObject *setterobj;
     if (setter)
     {
-        setterobj = GeneratePropertyOp(cx, obj, idval, 1, name, setter);
+        setterobj = GeneratePropertyOp(cx, obj, id, 1, setter);
         if(!setterobj)
             return JS_FALSE;
         roots[1] = OBJECT_TO_JSVAL(setterobj);
@@ -229,22 +251,15 @@ ReifyPropertyOps(JSContext *cx, JSObject *obj, jsval idval, jsid interned_id,
         *getterobjp = getterobj;
     if(setterobjp)
         *setterobjp = setterobj;
-    return JS_DefinePropertyById(cx, obj, interned_id, JSVAL_VOID,
+    return JS_DefinePropertyById(cx, obj, id, JSVAL_VOID,
                                  JS_DATA_TO_FUNC_PTR(JSPropertyOp, getterobj),
-                                 JS_DATA_TO_FUNC_PTR(JSPropertyOp, setterobj),
+                                 JS_DATA_TO_FUNC_PTR(JSStrictPropertyOp, setterobj),
                                  attrs);
 }
 
 static JSBool
 LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, uintN argc, jsval *vp)
 {
-    uintN attrs;
-    JSBool found;
-    JSPropertyOp getter, setter;
-    JSObject *obj2;
-    jsid interned_id;
-    jsval v;
-
     XPC_QS_ASSERT_CONTEXT_OK(cx);
 
     if(argc == 0)
@@ -254,23 +269,18 @@ LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, uintN argc, jsval *vp)
     }
 
     JSObject *obj = JS_THIS_OBJECT(cx, vp);
-    if (!obj)
+    if(!obj)
         return JS_FALSE;
-    jsval idval = JS_ARGV(cx, vp)[0];
 
-    const char *name = JSVAL_IS_STRING(idval)
-                       ? JS_GetStringBytes(JSVAL_TO_STRING(idval))
-                       : nsnull;
-    if(!JS_ValueToId(cx, idval, &interned_id) ||
-       !JS_LookupPropertyWithFlagsById(cx, obj, interned_id,
-                                       JSRESOLVE_QUALIFIED, &obj2, &v) ||
-       (obj2 &&
-        !JS_GetPropertyAttrsGetterAndSetterById(cx, obj2, interned_id, &attrs,
-                                                &found, &getter, &setter)))
+    jsval idval = JS_ARGV(cx, vp)[0];
+    jsid id;
+    JSPropertyDescriptor desc;
+    if(!JS_ValueToId(cx, idval, &id) ||
+       !JS_GetPropertyDescriptorById(cx, obj, id, JSRESOLVE_QUALIFIED, &desc))
         return JS_FALSE;
 
     // No property at all means no getters or setters possible.
-    if(!obj2 || !found)
+    if(!desc.obj)
     {
         JS_SET_RVAL(cx, vp, JSVAL_VOID);
         return JS_TRUE;
@@ -279,19 +289,19 @@ LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, uintN argc, jsval *vp)
     // Inline obj_lookup[GS]etter here.
     if(wantGetter)
     {
-        if(attrs & JSPROP_GETTER)
+        if(desc.attrs & JSPROP_GETTER)
         {
             JS_SET_RVAL(cx, vp,
-                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, getter)));
+                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, desc.getter)));
             return JS_TRUE;
         }
     }
     else
     {
-        if(attrs & JSPROP_SETTER)
+        if(desc.attrs & JSPROP_SETTER)
         {
             JS_SET_RVAL(cx, vp,
-                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, setter)));
+                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, desc.setter)));
             return JS_TRUE;
         }
     }
@@ -300,22 +310,26 @@ LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, uintN argc, jsval *vp)
     // ensuring that we have an XPConnect prototype object ensures that
     // we are only going to expose quickstubbed properties to script.
     // Also be careful not to overwrite existing properties!
-    if(!name ||
-       !IS_PROTO_CLASS(STOBJ_GET_CLASS(obj2)) ||
-       (attrs & (JSPROP_GETTER | JSPROP_SETTER)) ||
-       !(getter || setter))
+
+    if(!JSID_IS_STRING(id) ||
+       !IS_PROTO_CLASS(desc.obj->getClass()) ||
+       (desc.attrs & (JSPROP_GETTER | JSPROP_SETTER)) ||
+       !(desc.getter || desc.setter) ||
+       desc.setter == desc.obj->getJSClass()->setProperty)
     {
         JS_SET_RVAL(cx, vp, JSVAL_VOID);
         return JS_TRUE;
     }
 
     JSObject *getterobj, *setterobj;
-    if(!ReifyPropertyOps(cx, obj, idval, interned_id, name, getter, setter,
+    if(!ReifyPropertyOps(cx, desc.obj, id, desc.attrs, desc.getter, desc.setter,
                          &getterobj, &setterobj))
+    {
         return JS_FALSE;
+    }
 
     JSObject *wantedobj = wantGetter ? getterobj : setterobj;
-    v = wantedobj ? OBJECT_TO_JSVAL(wantedobj) : JSVAL_VOID;
+    jsval v = wantedobj ? OBJECT_TO_JSVAL(wantedobj) : JSVAL_VOID;
     JS_SET_RVAL(cx, vp, v);
     return JS_TRUE;
 }
@@ -332,36 +346,32 @@ SharedLookupSetter(JSContext *cx, uintN argc, jsval *vp)
     return LookupGetterOrSetter(cx, PR_FALSE, argc, vp);
 }
 
-// XXX Hack! :-/
-JS_FRIEND_API(JSBool) js_obj_defineGetter(JSContext *cx, uintN argc, jsval *vp);
-JS_FRIEND_API(JSBool) js_obj_defineSetter(JSContext *cx, uintN argc, jsval *vp);
-
 static JSBool
 DefineGetterOrSetter(JSContext *cx, uintN argc, JSBool wantGetter, jsval *vp)
 {
     uintN attrs;
     JSBool found;
-    JSPropertyOp getter, setter;
+    JSPropertyOp getter;
+    JSStrictPropertyOp setter;
     JSObject *obj2;
     jsval v;
-    jsid interned_id;
+    jsid id;
 
     XPC_QS_ASSERT_CONTEXT_OK(cx);
     JSObject *obj = JS_THIS_OBJECT(cx, vp);
     if (!obj)
         return JS_FALSE;
-    JSFastNative forward = wantGetter ? js_obj_defineGetter : js_obj_defineSetter;
-    jsval id = (argc >= 1) ? JS_ARGV(cx, vp)[0] : JSVAL_VOID;
-    if(!JSVAL_IS_STRING(id))
+    JSNative forward = wantGetter ? Jsvalify(js_obj_defineGetter)
+                                  : Jsvalify(js_obj_defineSetter);
+    jsval idval = (argc >= 1) ? JS_ARGV(cx, vp)[0] : JSVAL_VOID;
+    if(!JSVAL_IS_STRING(idval))
         return forward(cx, argc, vp);
-    JSString *str = JSVAL_TO_STRING(id);
 
-    const char *name = JS_GetStringBytes(str);
-    if(!JS_ValueToId(cx, id, &interned_id) ||
-       !JS_LookupPropertyWithFlagsById(cx, obj, interned_id,
+    if(!JS_ValueToId(cx, idval, &id) ||
+       !JS_LookupPropertyWithFlagsById(cx, obj, id,
                                        JSRESOLVE_QUALIFIED, &obj2, &v) ||
        (obj2 &&
-        !JS_GetPropertyAttrsGetterAndSetterById(cx, obj2, interned_id, &attrs,
+        !JS_GetPropertyAttrsGetterAndSetterById(cx, obj2, id, &attrs,
                                                 &found, &getter, &setter)))
         return JS_FALSE;
 
@@ -370,12 +380,11 @@ DefineGetterOrSetter(JSContext *cx, uintN argc, JSBool wantGetter, jsval *vp)
     if(!obj2 ||
        (attrs & (JSPROP_GETTER | JSPROP_SETTER)) ||
        !(getter || setter) ||
-       !IS_PROTO_CLASS(STOBJ_GET_CLASS(obj2)))
+       !IS_PROTO_CLASS(obj2->getClass()))
         return forward(cx, argc, vp);
 
     // Reify the getter and setter...
-    if(!ReifyPropertyOps(cx, obj, id, interned_id, name, getter, setter,
-                         nsnull, nsnull))
+    if(!ReifyPropertyOps(cx, obj2, id, attrs, getter, setter, nsnull, nsnull))
         return JS_FALSE;
 
     return forward(cx, argc, vp);
@@ -441,7 +450,7 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, uintN flags,
                         if(!JS_DefineFunction(
                                cx, proto, fs->name,
                                reinterpret_cast<JSNative>(fs->native),
-                               fs->arity, flags | JSFUN_FAST_NATIVE))
+                               fs->arity, flags))
                             return JS_FALSE;
                     }
                 }
@@ -453,8 +462,7 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, uintN flags,
                     {
                         if(!JS_DefineFunction(
                                cx, proto, ts->name, ts->native, ts->arity,
-                               flags | JSFUN_FAST_NATIVE | JSFUN_STUB_GSOPS |
-                                       JSFUN_TRCINFO))
+                               flags | JSFUN_STUB_GSOPS | JSFUN_TRCINFO))
                             return JS_FALSE;
                     }
                 }
@@ -499,10 +507,7 @@ xpc_qsThrow(JSContext *cx, nsresult rv)
  * rather than "[nsIDOMNode.appendChild]".
  */
 static void
-GetMemberInfo(JSObject *obj,
-              jsval memberId,
-              const char **ifaceName,
-              const char **memberName)
+GetMemberInfo(JSObject *obj, jsid memberId, const char **ifaceName)
 {
     // Get the interface name.  From DefinePropertyIfFound (in
     // xpcwrappednativejsops.cpp) and XPCThrower::Verbosify.
@@ -511,9 +516,8 @@ GetMemberInfo(JSObject *obj,
     // but this code often produces a more specific error message, e.g.
     *ifaceName = "Unknown";
 
-    NS_ASSERTION(IS_WRAPPER_CLASS(STOBJ_GET_CLASS(obj)) ||
-                 STOBJ_GET_CLASS(obj) == &XPC_WN_Tearoff_JSClass ||
-                 IS_SLIM_WRAPPER(obj),
+    NS_ASSERTION(IS_WRAPPER_CLASS(obj->getClass()) ||
+                 obj->getClass() == &XPC_WN_Tearoff_JSClass,
                  "obj must be a wrapper");
     XPCWrappedNativeProto *proto;
     if(IS_SLIM_WRAPPER(obj))
@@ -537,31 +541,27 @@ GetMemberInfo(JSObject *obj,
                 *ifaceName = iface->GetNameString();
         }
     }
-
-    *memberName = (JSVAL_IS_STRING(memberId)
-                   ? JS_GetStringBytes(JSVAL_TO_STRING(memberId))
-                   : "unknown");
 }
 
 static void
-GetMethodInfo(JSContext *cx,
-              jsval *vp,
-              const char **ifaceName,
-              const char **memberName)
+GetMethodInfo(JSContext *cx, jsval *vp, const char **ifaceNamep, jsid *memberIdp)
 {
     JSObject *funobj = JSVAL_TO_OBJECT(JS_CALLEE(cx, vp));
     NS_ASSERTION(JS_ObjectIsFunction(cx, funobj),
-                 "JSFastNative callee should be Function object");
+                 "JSNative callee should be Function object");
     JSString *str = JS_GetFunctionId((JSFunction *) JS_GetPrivate(cx, funobj));
-    jsval methodId = str ? STRING_TO_JSVAL(str) : JSVAL_NULL;
-
-    GetMemberInfo(JSVAL_TO_OBJECT(vp[1]), methodId, ifaceName, memberName);
+    jsid methodId = str ? INTERNED_STRING_TO_JSID(cx, str) : JSID_VOID;
+    GetMemberInfo(JSVAL_TO_OBJECT(vp[1]), methodId, ifaceNamep);
+    *memberIdp = methodId;
 }
 
 static JSBool
 ThrowCallFailed(JSContext *cx, nsresult rv,
-                const char *ifaceName, const char *memberName)
+                const char *ifaceName, jsid memberId, const char *memberName)
 {
+    /* Only one of memberId or memberName should be given. */
+    JS_ASSERT(JSID_IS_VOID(memberId) != !memberName);
+    
     // From XPCThrower::ThrowBadResult.
     char* sz;
     const char* format;
@@ -585,6 +585,12 @@ ThrowCallFailed(JSContext *cx, nsresult rv,
         format = "";
     }
 
+    JSAutoByteString memberNameBytes;
+    if (!memberName) {
+        memberName = JSID_IS_STRING(memberId)
+                     ? memberNameBytes.encode(cx, JSID_TO_STRING(memberId))
+                     : "unknown";
+    }
     if(nsXPCException::NameAndFormatForNSResult(rv, &name, nsnull)
         && name)
     {
@@ -607,19 +613,20 @@ ThrowCallFailed(JSContext *cx, nsresult rv,
 
 JSBool
 xpc_qsThrowGetterSetterFailed(JSContext *cx, nsresult rv, JSObject *obj,
-                              jsval memberId)
+                              jsid memberId)
 {
-    const char *ifaceName, *memberName;
-    GetMemberInfo(obj, memberId, &ifaceName, &memberName);
-    return ThrowCallFailed(cx, rv, ifaceName, memberName);
+    const char *ifaceName;
+    GetMemberInfo(obj, memberId, &ifaceName);
+    return ThrowCallFailed(cx, rv, ifaceName, memberId, NULL);
 }
 
 JSBool
 xpc_qsThrowMethodFailed(JSContext *cx, nsresult rv, jsval *vp)
 {
-    const char *ifaceName, *memberName;
-    GetMethodInfo(cx, vp, &ifaceName, &memberName);
-    return ThrowCallFailed(cx, rv, ifaceName, memberName);
+    const char *ifaceName;
+    jsid memberId;
+    GetMethodInfo(cx, vp, &ifaceName, &memberId);
+    return ThrowCallFailed(cx, rv, ifaceName, memberId, NULL);
 }
 
 JSBool
@@ -634,13 +641,16 @@ xpc_qsThrowMethodFailedWithDetails(JSContext *cx, nsresult rv,
                                    const char *ifaceName,
                                    const char *memberName)
 {
-    ThrowCallFailed(cx, rv, ifaceName, memberName);
+    ThrowCallFailed(cx, rv, ifaceName, JSID_VOID, memberName);
 }
 
 static void
-ThrowBadArg(JSContext *cx, nsresult rv,
-            const char *ifaceName, const char *memberName, uintN paramnum)
+ThrowBadArg(JSContext *cx, nsresult rv, const char *ifaceName,
+            jsid memberId, const char *memberName, uintN paramnum)
 {
+    /* Only one memberId or memberName should be given. */
+    JS_ASSERT(JSID_IS_VOID(memberId) != !memberName);
+
     // From XPCThrower::ThrowBadParam.
     char* sz;
     const char* format;
@@ -648,6 +658,12 @@ ThrowBadArg(JSContext *cx, nsresult rv,
     if(!nsXPCException::NameAndFormatForNSResult(rv, nsnull, &format))
         format = "";
 
+    JSAutoByteString memberNameBytes;
+    if (!memberName) {
+        memberName = JSID_IS_STRING(memberId)
+                     ? memberNameBytes.encode(cx, JSID_TO_STRING(memberId))
+                     : "unknown";
+    }
     sz = JS_smprintf("%s arg %u [%s.%s]",
                      format, (unsigned int) paramnum, ifaceName, memberName);
 
@@ -660,9 +676,10 @@ ThrowBadArg(JSContext *cx, nsresult rv,
 void
 xpc_qsThrowBadArg(JSContext *cx, nsresult rv, jsval *vp, uintN paramnum)
 {
-    const char *ifaceName, *memberName;
-    GetMethodInfo(cx, vp, &ifaceName, &memberName);
-    ThrowBadArg(cx, rv, ifaceName, memberName, paramnum);
+    const char *ifaceName;
+    jsid memberId;
+    GetMethodInfo(cx, vp, &ifaceName, &memberId);
+    ThrowBadArg(cx, rv, ifaceName, memberId, NULL, paramnum);
 }
 
 void
@@ -675,128 +692,91 @@ void
 xpc_qsThrowBadArgWithDetails(JSContext *cx, nsresult rv, uintN paramnum,
                              const char *ifaceName, const char *memberName)
 {
-    ThrowBadArg(cx, rv, ifaceName, memberName, paramnum);
+    ThrowBadArg(cx, rv, ifaceName, JSID_VOID, memberName, paramnum);
 }
 
 void
 xpc_qsThrowBadSetterValue(JSContext *cx, nsresult rv,
-                          JSObject *obj, jsval propId)
+                          JSObject *obj, jsid propId)
 {
-    const char *ifaceName, *memberName;
-    GetMemberInfo(obj, propId, &ifaceName, &memberName);
-    ThrowBadArg(cx, rv, ifaceName, memberName, 0);
+    const char *ifaceName;
+    GetMemberInfo(obj, propId, &ifaceName);
+    ThrowBadArg(cx, rv, ifaceName, propId, NULL, 0);
 }
 
-xpc_qsDOMString::xpc_qsDOMString(JSContext *cx, jsval *pval)
+JSBool
+xpc_qsGetterOnlyPropertyStub(JSContext *cx, JSObject *obj, jsid id, JSBool strict, jsval *vp)
 {
+    return JS_ReportErrorFlagsAndNumber(cx,
+                                        JSREPORT_WARNING | JSREPORT_STRICT |
+                                        JSREPORT_STRICT_MODE_ERROR,
+                                        js_GetErrorMessage, NULL,
+                                        JSMSG_GETTER_ONLY);
+}
+
+xpc_qsDOMString::xpc_qsDOMString(JSContext *cx, jsval v, jsval *pval,
+                                 StringificationBehavior nullBehavior,
+                                 StringificationBehavior undefinedBehavior)
+{
+    typedef implementation_type::char_traits traits;
     // From the T_DOMSTRING case in XPCConvert::JSData2Native.
-    typedef implementation_type::char_traits traits;
-    jsval v;
-    JSString *s;
-    const PRUnichar *chars;
+    JSString *s = InitOrStringify<traits>(cx, v, pval, nullBehavior,
+                                          undefinedBehavior);
+    if (!s)
+        return;
+
     size_t len;
+    const jschar *chars = JS_GetStringCharsZAndLength(cx, s, &len);
+    if (!chars)
+        return;
 
-    v = *pval;
-    if(JSVAL_IS_STRING(v))
-    {
-        s = JSVAL_TO_STRING(v);
-    }
-    else
-    {
-        if(JSVAL_IS_NULL(v))
-        {
-            (new(mBuf) implementation_type(
-                traits::sEmptyBuffer, PRUint32(0)))->SetIsVoid(PR_TRUE);
-            mValid = JS_TRUE;
-            return;
-        }
-
-        s = JS_ValueToString(cx, v);
-        if(!s)
-        {
-            mValid = JS_FALSE;
-            return;
-        }
-        *pval = STRING_TO_JSVAL(s);  // Root the new string.
-    }
-
-    len = JS_GetStringLength(s);
-    chars = (len == 0 ? traits::sEmptyBuffer : (const PRUnichar*)JS_GetStringChars(s));
     new(mBuf) implementation_type(chars, len);
     mValid = JS_TRUE;
 }
 
-xpc_qsAString::xpc_qsAString(JSContext *cx, jsval *pval)
+xpc_qsACString::xpc_qsACString(JSContext *cx, jsval v, jsval *pval,
+                               StringificationBehavior nullBehavior,
+                               StringificationBehavior undefinedBehavior)
 {
-    // From the T_ASTRING case in XPCConvert::JSData2Native.
     typedef implementation_type::char_traits traits;
-    jsval v;
-    JSString *s;
-    const PRUnichar *chars;
-    size_t len;
-
-    v = *pval;
-    if(JSVAL_IS_STRING(v))
-    {
-        s = JSVAL_TO_STRING(v);
-    }
-    else
-    {
-        if(JSVAL_IS_NULL(v) || JSVAL_IS_VOID(v))
-        {
-            (new(mBuf) implementation_type(
-                traits::sEmptyBuffer, PRUint32(0)))->SetIsVoid(PR_TRUE);
-            mValid = JS_TRUE;
-            return;
-        }
-
-        s = JS_ValueToString(cx, v);
-        if(!s)
-        {
-            mValid = JS_FALSE;
-            return;
-        }
-        *pval = STRING_TO_JSVAL(s);  // Root the new string.
-    }
-
-    len = JS_GetStringLength(s);
-    chars = (len == 0 ? traits::sEmptyBuffer : (const PRUnichar*)JS_GetStringChars(s));
-    new(mBuf) implementation_type(chars, len);
-    mValid = JS_TRUE;
-}
-
-xpc_qsACString::xpc_qsACString(JSContext *cx, jsval *pval)
-{
     // From the T_CSTRING case in XPCConvert::JSData2Native.
-    jsval v;
-    JSString *s;
+    JSString *s = InitOrStringify<traits>(cx, v, pval, nullBehavior,
+                                          undefinedBehavior);
+    if (!s)
+        return;
 
-    v = *pval;
-    if(JSVAL_IS_STRING(v))
+    size_t len = JS_GetStringEncodingLength(cx, s);
+    if(len == size_t(-1))
     {
-        s = JSVAL_TO_STRING(v);
-    }
-    else
-    {
-        if(JSVAL_IS_NULL(v) || JSVAL_IS_VOID(v))
-        {
-            (new(mBuf) implementation_type())->SetIsVoid(PR_TRUE);
-            mValid = JS_TRUE;
-            return;
-        }
-
-        s = JS_ValueToString(cx, v);
-        if(!s)
-        {
-            mValid = JS_FALSE;
-            return;
-        }
-        *pval = STRING_TO_JSVAL(s);  // Root the new string.
+        mValid = JS_FALSE;
+        return;
     }
 
-    const char *bytes = JS_GetStringBytes(s);
-    size_t len = JS_GetStringLength(s);
-    new(mBuf) implementation_type(bytes, len);
+    JSAutoByteString bytes(cx, s);
+    if(!bytes)
+    {
+        mValid = JS_FALSE;
+        return;
+    }
+
+    new(mBuf) implementation_type(bytes.ptr(), len);
+    mValid = JS_TRUE;
+}
+
+xpc_qsAUTF8String::xpc_qsAUTF8String(JSContext *cx, jsval v, jsval *pval)
+{
+    typedef nsCharTraits<PRUnichar> traits;
+    // From the T_UTF8STRING  case in XPCConvert::JSData2Native.
+    JSString *s = InitOrStringify<traits>(cx, v, pval, eNull, eNull);
+    if (!s)
+        return;
+
+    size_t len;
+    const PRUnichar *chars = JS_GetStringCharsZAndLength(cx, s, &len);
+    if (!chars)
+        return;
+
+    new(mBuf) implementation_type(chars, len);
     mValid = JS_TRUE;
 }
 
@@ -831,8 +811,9 @@ getNative(nsISupports *idobj,
     return rv;
 }
 
-static nsresult
-getNativeFromWrapper(XPCWrappedNative *wrapper,
+inline nsresult
+getNativeFromWrapper(JSContext *cx,
+                     XPCWrappedNative *wrapper,
                      const nsIID &iid,
                      void **ppThis,
                      nsISupports **pThisRef,
@@ -843,49 +824,68 @@ getNativeFromWrapper(XPCWrappedNative *wrapper,
 }
 
 
-JSBool
-xpc_qsUnwrapThisImpl(JSContext *cx,
-                     JSObject *obj,
-                     JSObject *callee,
-                     const nsIID &iid,
-                     void **ppThis,
-                     nsISupports **pThisRef,
-                     jsval *vp,
-                     XPCLazyCallContext *lccx)
+nsresult
+getWrapper(JSContext *cx,
+           JSObject *obj,
+           JSObject *callee,
+           XPCWrappedNative **wrapper,
+           JSObject **cur,
+           XPCWrappedNativeTearOff **tearoff)
+{
+    if(XPCWrapper::IsSecurityWrapper(obj) &&
+       !(obj = XPCWrapper::Unwrap(cx, obj)))
+    {
+        return NS_ERROR_XPC_SECURITY_MANAGER_VETO;
+    }
+
+    *cur = obj;
+    *tearoff = nsnull;
+
+    *wrapper =
+        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj, callee, cur,
+                                                     tearoff);
+
+    return NS_OK;
+}
+
+nsresult
+castNative(JSContext *cx,
+           XPCWrappedNative *wrapper,
+           JSObject *cur,
+           XPCWrappedNativeTearOff *tearoff,
+           const nsIID &iid,
+           void **ppThis,
+           nsISupports **pThisRef,
+           jsval *vp,
+           XPCLazyCallContext *lccx)
 {
-    JSObject *cur = obj;
-    XPCWrappedNativeTearOff *tearoff;
-    XPCWrappedNative *wrapper =
-        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj, callee, &cur,
-                                                     &tearoff);
     if(wrapper)
     {
-        nsresult rv = getNativeFromWrapper(wrapper, iid, ppThis, pThisRef, vp);
-        if(NS_SUCCEEDED(rv))
-        {
-            if(lccx)
-                lccx->SetWrapper(cur, wrapper, tearoff);
-            
-            return JS_TRUE;
-        }
+        nsresult rv = getNativeFromWrapper(cx,wrapper, iid, ppThis, pThisRef,
+                                           vp);
+
+        if(lccx && NS_SUCCEEDED(rv))
+            lccx->SetWrapper(wrapper, tearoff);
+
         if(rv != NS_ERROR_NO_INTERFACE)
-            return xpc_qsThrow(cx, rv);
+            return rv;
     }
     else if(cur)
     {
+        NS_ABORT_IF_FALSE(IS_SLIM_WRAPPER(cur), "should be a slim wrapper");
         nsISupports *native = static_cast<nsISupports*>(xpc_GetJSPrivate(cur));
         if(NS_SUCCEEDED(getNative(native, GetOffsetsFromSlimWrapper(cur),
                                   cur, iid, ppThis, pThisRef, vp)))
         {
             if(lccx)
-                lccx->SetWrapper(cur, nsnull, nsnull);
+                lccx->SetWrapper(cur);
 
-            return JS_TRUE;
+            return NS_OK;
         }
     }
 
     *pThisRef = nsnull;
-    return xpc_qsThrow(cx, NS_ERROR_XPC_BAD_OP_ON_WN_PROTO);
+    return NS_ERROR_XPC_BAD_OP_ON_WN_PROTO;
 }
 
 JSBool
@@ -907,6 +907,29 @@ xpc_qsUnwrapThisFromCcxImpl(XPCCallContext &ccx,
     return JS_TRUE;
 }
 
+JSObject*
+xpc_qsUnwrapObj(jsval v, nsISupports **ppArgRef, nsresult *rv)
+{
+    if(JSVAL_IS_VOID(v) || JSVAL_IS_NULL(v))
+    {
+        *ppArgRef = nsnull;
+        *rv = NS_OK;
+        return nsnull;
+    }
+
+    if(!JSVAL_IS_OBJECT(v))
+    {
+        *ppArgRef = nsnull;
+        *rv = ((JSVAL_IS_INT(v) && JSVAL_TO_INT(v) == 0)
+              ? NS_ERROR_XPC_BAD_CONVERT_JS_ZERO_ISNOT_NULL
+              : NS_ERROR_XPC_BAD_CONVERT_JS);
+        return nsnull;
+    }
+
+    *rv = NS_OK;
+    return JSVAL_TO_OBJECT(v);
+}
+
 nsresult
 xpc_qsUnwrapArgImpl(JSContext *cx,
                     jsval v,
@@ -915,41 +938,25 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
                     nsISupports **ppArgRef,
                     jsval *vp)
 {
-    // From XPCConvert::JSData2Native
-    if(JSVAL_IS_VOID(v) || JSVAL_IS_NULL(v))
+    nsresult rv;
+    JSObject *src = xpc_qsUnwrapObj(v, ppArgRef, &rv);
+    if(!src)
     {
         *ppArg = nsnull;
-        *ppArgRef = nsnull;
-        return NS_OK;
+
+        return rv;
     }
 
-    if(!JSVAL_IS_OBJECT(v))
-    {
-        *ppArgRef = nsnull;
-        return ((JSVAL_IS_INT(v) && JSVAL_TO_INT(v) == 0)
-                ? NS_ERROR_XPC_BAD_CONVERT_JS_ZERO_ISNOT_NULL
-                : NS_ERROR_XPC_BAD_CONVERT_JS);
-    }
-    JSObject *src = JSVAL_TO_OBJECT(v);
+    XPCWrappedNative *wrapper;
+    XPCWrappedNativeTearOff *tearoff;
+    JSObject *obj2;
+    rv = getWrapper(cx, src, nsnull, &wrapper, &obj2, &tearoff);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-    if(IS_SLIM_WRAPPER(src))
+    if(wrapper || obj2)
     {
-        nsISupports *iface = static_cast<nsISupports*>(xpc_GetJSPrivate(src));
-        if(NS_FAILED(getNative(iface, GetOffsetsFromSlimWrapper(src),
-                               src, iid, ppArg, ppArgRef, vp)))
-            return NS_ERROR_XPC_BAD_CONVERT_JS;
-        return NS_OK;
-    }
-
-    // From XPCConvert::JSObject2NativeInterface
-    XPCWrappedNative* wrappedNative =
-        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, src);
-    nsISupports *iface;
-    if(wrappedNative)
-    {
-        iface = wrappedNative->GetIdentityObject();
-        if(NS_FAILED(getNativeFromWrapper(wrappedNative, iid, ppArg, ppArgRef,
-                                          vp)))
+        if(NS_FAILED(castNative(cx, wrapper, obj2, tearoff, iid, ppArg,
+                                ppArgRef, vp, nsnull)))
             return NS_ERROR_XPC_BAD_CONVERT_JS;
         return NS_OK;
     }
@@ -965,9 +972,8 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
         return NS_ERROR_XPC_BAD_CONVERT_JS;
     }
 
-    // Does the JSObject have 'nsISupportness'?
-    // XXX hmm, I wonder if this matters anymore with no
-    // oldstyle DOM objects around.
+    // Try to unwrap a slim wrapper.
+    nsISupports *iface;
     if(XPCConvert::GetISupportsFromJSObject(src, &iface))
     {
         if(!iface || NS_FAILED(iface->QueryInterface(iid, ppArg)))
@@ -988,10 +994,10 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
         return NS_ERROR_XPC_BAD_CONVERT_JS;
     }
 
-    nsXPCWrappedJS *wrapper;
-    nsresult rv =
-        nsXPCWrappedJS::GetNewOrUsed(ccx, src, iid, nsnull, &wrapper);
-    if(NS_FAILED(rv) || !wrapper)
+    nsRefPtr<nsXPCWrappedJS> wrappedJS;
+    rv = nsXPCWrappedJS::GetNewOrUsed(ccx, src, iid, nsnull,
+                                      getter_AddRefs(wrappedJS));
+    if(NS_FAILED(rv) || !wrappedJS)
     {
         *ppArgRef = nsnull;
         return rv;
@@ -1001,20 +1007,40 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
     // the right thing for the various 'special' interfaces; e.g.
     // nsIPropertyBag. We must use AggregatedQueryInterface in cases where
     // there is an outer to avoid nasty recursion.
-    rv = wrapper->QueryInterface(iid, ppArg);
+    rv = wrappedJS->QueryInterface(iid, ppArg);
     if(NS_SUCCEEDED(rv))
     {
         *ppArgRef = static_cast<nsISupports*>(*ppArg);
-        *vp = OBJECT_TO_JSVAL(wrapper->GetJSObject());
+        *vp = OBJECT_TO_JSVAL(wrappedJS->GetJSObject());
     }
-    NS_RELEASE(wrapper);
     return rv;
 }
 
 JSBool
-xpc_qsJsvalToCharStr(JSContext *cx, jsval *pval, char **pstr)
+xpc_qsJsvalToCharStr(JSContext *cx, jsval v, JSAutoByteString *bytes)
+{
+    JSString *str;
+
+    JS_ASSERT(!bytes->ptr());
+    if(JSVAL_IS_STRING(v))
+    {
+        str = JSVAL_TO_STRING(v);
+    }
+    else if(JSVAL_IS_VOID(v) || JSVAL_IS_NULL(v))
+    {
+        return true;
+    }
+    else
+    {
+        if(!(str = JS_ValueToString(cx, v)))
+            return false;
+    }
+    return !!bytes->encode(cx, str);
+}
+
+JSBool
+xpc_qsJsvalToWcharStr(JSContext *cx, jsval v, jsval *pval, PRUnichar **pstr)
 {
-    jsval v = *pval;
     JSString *str;
 
     if(JSVAL_IS_STRING(v))
@@ -1033,38 +1059,17 @@ xpc_qsJsvalToCharStr(JSContext *cx, jsval *pval, char **pstr)
         *pval = STRING_TO_JSVAL(str);  // Root the new string.
     }
 
-    *pstr = JS_GetStringBytes(str);
+    const jschar *chars = JS_GetStringCharsZ(cx, str);
+    if (!chars)
+        return JS_FALSE;
+
+    // XXXbz this is casting away constness too...  That seems like a bad idea.
+    *pstr = const_cast<jschar *>(chars);
     return JS_TRUE;
 }
 
 JSBool
-xpc_qsJsvalToWcharStr(JSContext *cx, jsval *pval, PRUnichar **pstr)
-{
-    jsval v = *pval;
-    JSString *str;
-
-    if(JSVAL_IS_STRING(v))
-    {
-        str = JSVAL_TO_STRING(v);
-    }
-    else if(JSVAL_IS_VOID(v) || JSVAL_IS_NULL(v))
-    {
-        *pstr = NULL;
-        return JS_TRUE;
-    }
-    else
-    {
-        if(!(str = JS_ValueToString(cx, v)))
-            return JS_FALSE;
-        *pval = STRING_TO_JSVAL(str);  // Root the new string.
-    }
-
-    *pstr = (PRUnichar*)JS_GetStringChars(str);
-    return JS_TRUE;
-}
-
-JSBool
-xpc_qsStringToJsval(JSContext *cx, const nsAString &str, jsval *rval)
+xpc_qsStringToJsval(JSContext *cx, nsString &str, jsval *rval)
 {
     // From the T_DOMSTRING case in XPCConvert::NativeData2JS.
     if(str.IsVoid())
@@ -1073,24 +1078,55 @@ xpc_qsStringToJsval(JSContext *cx, const nsAString &str, jsval *rval)
         return JS_TRUE;
     }
 
-    jsval jsstr = XPCStringConvert::ReadableToJSVal(cx, str);
-    if(!jsstr)
+    nsStringBuffer* sharedBuffer;
+    jsval jsstr = XPCStringConvert::ReadableToJSVal(cx, str, &sharedBuffer);
+    if (JSVAL_IS_NULL(jsstr))
         return JS_FALSE;
     *rval = jsstr;
+    if (sharedBuffer)
+    {
+        // The string was shared but ReadableToJSVal didn't addref it.
+        // Move the ownership from str to jsstr.
+        str.ForgetSharedBuffer();
+    }
     return JS_TRUE;
 }
 
 JSBool
-xpc_qsXPCOMObjectToJsval(XPCLazyCallContext &lccx, nsISupports *p,
-                         nsWrapperCache *cache, const nsIID *iid,
-                         XPCNativeInterface **iface, jsval *rval)
+xpc_qsStringToJsstring(JSContext *cx, nsString &str, JSString **rval)
 {
+    // From the T_DOMSTRING case in XPCConvert::NativeData2JS.
+    if(str.IsVoid())
+    {
+        *rval = nsnull;
+        return JS_TRUE;
+    }
+
+    nsStringBuffer* sharedBuffer;
+    jsval jsstr = XPCStringConvert::ReadableToJSVal(cx, str, &sharedBuffer);
+    if(JSVAL_IS_NULL(jsstr))
+        return JS_FALSE;
+    *rval = JSVAL_TO_STRING(jsstr);
+    if (sharedBuffer)
+    {
+        // The string was shared but ReadableToJSVal didn't addref it.
+        // Move the ownership from str to jsstr.
+        str.ForgetSharedBuffer();
+    }
+    return JS_TRUE;
+}
+
+JSBool
+xpc_qsXPCOMObjectToJsval(XPCLazyCallContext &lccx, qsObjectHelper &aHelper,
+                         const nsIID *iid, XPCNativeInterface **iface,
+                         jsval *rval)
+{
+    NS_PRECONDITION(iface, "Who did that and why?");
+
     // From the T_INTERFACE case in XPCConvert::NativeData2JS.
     // This is one of the slowest things quick stubs do.
 
     JSContext *cx = lccx.GetJSContext();
-    if(!iface)
-        return xpc_qsThrow(cx, NS_ERROR_XPC_BAD_CONVERT_NATIVE);
 
     // XXX The OBJ_IS_NOT_GLOBAL here is not really right. In
     // fact, this code is depending on the fact that the
@@ -1099,10 +1135,9 @@ xpc_qsXPCOMObjectToJsval(XPCLazyCallContext &lccx, nsISupports *p,
     // creating a new XPCNativeScriptableShared.
 
     nsresult rv;
-    if(!XPCConvert::NativeInterface2JSObject(lccx, rval, nsnull, p,
-                                             iid, iface, cache,
-                                             lccx.GetCurrentJSObject(), PR_TRUE,
-                                             OBJ_IS_NOT_GLOBAL, &rv))
+    if(!XPCConvert::NativeInterface2JSObject(lccx, rval, nsnull,
+                                             aHelper, iid, iface,
+                                             PR_TRUE, OBJ_IS_NOT_GLOBAL, &rv))
     {
         // I can't tell if NativeInterface2JSObject throws JS exceptions
         // or not.  This is a sloppy stab at the right semantics; the
@@ -1114,8 +1149,8 @@ xpc_qsXPCOMObjectToJsval(XPCLazyCallContext &lccx, nsISupports *p,
 
 #ifdef DEBUG
     JSObject* jsobj = JSVAL_TO_OBJECT(*rval);
-    if(jsobj && !STOBJ_GET_PARENT(jsobj))
-        NS_ASSERTION(STOBJ_GET_CLASS(jsobj)->flags & JSCLASS_IS_GLOBAL,
+    if(jsobj && !jsobj->getParent())
+        NS_ASSERTION(jsobj->getClass()->flags & JSCLASS_IS_GLOBAL,
                      "Why did we recreate this wrapper?");
 #endif
 
@@ -1132,9 +1167,7 @@ xpc_qsVariantToJsval(XPCLazyCallContext &lccx,
     if(p)
     {
         nsresult rv;
-        JSBool ok = XPCVariant::VariantDataToJS(lccx, p,
-                                                lccx.GetCurrentJSObject(),
-                                                &rv, rval);
+        JSBool ok = XPCVariant::VariantDataToJS(lccx, p, &rv, rval);
         if (!ok)
             xpc_qsThrow(lccx.GetJSContext(), rv);
         return ok;
