@@ -83,7 +83,11 @@
 #endif
 
 // Size to use for PLArena block allocations.
-static const size_t ARENA_PAGE_SIZE = 4096;
+// XXX: This should be 8192;  the subtracted elements are a hack that's
+// required to ensure the allocation requests are power-of-two-sized and thus
+// avoid lots of wasted memory caused by the heap allocator rounding up request
+// sizes.  Bug 676547 will fix it properly.
+static const size_t ARENA_PAGE_SIZE = 8192 - sizeof(PLArena) - PL_ARENA_CONST_ALIGN_MASK;
 
 // Freed memory is filled with a poison value, which we arrange to
 // form a pointer either to an always-unmapped region of the address
