@@ -43,10 +43,8 @@
  * as <frame>, <iframe>, and some <object>s
  */
 
-#ifdef MOZ_IPC
 #include "mozilla/layout/RenderFrameParent.h"
 using mozilla::layout::RenderFrameParent;
-#endif
 
 #include "nsSubDocumentFrame.h"
 #include "nsCOMPtr.h"
@@ -84,8 +82,6 @@ using mozilla::layout::RenderFrameParent;
 #include "nsWeakReference.h"
 #include "nsIDOMWindow.h"
 #include "nsIDOMDocument.h"
-#include "nsIRenderingContext.h"
-#include "nsIDOMNSHTMLDocument.h"
 #include "nsDisplayList.h"
 #include "nsUnicharUtils.h"
 #include "nsIScrollableFrame.h"
@@ -181,7 +177,7 @@ nsSubDocumentFrame::Init(nsIContent*     aContent,
   // really need it or not, and the inner view will get it as the
   // parent.
   if (!HasView()) {
-    rv = nsHTMLContainerFrame::CreateViewForFrame(this, PR_TRUE);
+    rv = nsContainerFrame::CreateViewForFrame(this, PR_TRUE);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -277,7 +273,6 @@ nsSubDocumentFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
   if (!mInnerView)
     return NS_OK;
 
-#ifdef MOZ_IPC
   nsFrameLoader* frameLoader = FrameLoader();
   if (frameLoader) {
     RenderFrameParent* rfp = frameLoader->GetCurrentRemoteFrame();
@@ -285,7 +280,6 @@ nsSubDocumentFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
       return rfp->BuildDisplayList(aBuilder, this, aDirtyRect, aLists);
     }
   }
-#endif
 
   nsIView* subdocView = mInnerView->GetFirstChild();
   if (!subdocView)
@@ -340,32 +334,12 @@ nsSubDocumentFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
   PRInt32 parentAPD = PresContext()->AppUnitsPerDevPixel();
   PRInt32 subdocAPD = presContext->AppUnitsPerDevPixel();
 
-  nsIFrame* subdocRootScrollFrame = presShell->GetRootScrollFrame();
-
   nsRect dirty;
   if (subdocRootFrame) {
-    if (presShell->UsingDisplayPort() && subdocRootScrollFrame) {
-      dirty = presShell->GetDisplayPort();
-
-      // The visual overflow rect of our viewport frame unfortunately may not
-      // intersect with the displayport of that frame. For example, the scroll
-      // offset of the frame may be (0, 0) so that the visual overflow rect
-      // is (0, 0, 800px, 500px) while the display port may have its top-left
-      // corner below y=500px.
-      //
-      // We have to force the frame to have a little faith and build a display
-      // list anyway. (see nsIFrame::BuildDisplayListForChild for the short-
-      // circuit code we are evading here).
-      //
-      subdocRootScrollFrame->AddStateBits(
-        NS_FRAME_FORCE_DISPLAY_LIST_DESCEND_INTO);
-
-    } else {
-      // get the dirty rect relative to the root frame of the subdoc
-      dirty = aDirtyRect + GetOffsetToCrossDoc(subdocRootFrame);
-      // and convert into the appunits of the subdoc
-      dirty = dirty.ConvertAppUnitsRoundOut(parentAPD, subdocAPD);
-    }
+    // get the dirty rect relative to the root frame of the subdoc
+    dirty = aDirtyRect + GetOffsetToCrossDoc(subdocRootFrame);
+    // and convert into the appunits of the subdoc
+    dirty = dirty.ConvertAppUnitsRoundOut(parentAPD, subdocAPD);
 
     aBuilder->EnterPresShell(subdocRootFrame, dirty);
   }
@@ -411,10 +385,7 @@ nsSubDocumentFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
       // Add the canvas background color to the bottom of the list. This
       // happens after we've built the list so that AddCanvasBackgroundColorItem
       // can monkey with the contents if necessary.
-      PRUint32 flags = nsIPresShell_MOZILLA_2_0_BRANCH::FORCE_DRAW;
-      if (presContext->IsRootContentDocument()) {
-        flags |= nsIPresShell_MOZILLA_2_0_BRANCH::ROOT_CONTENT_DOC_BG;
-      }
+      PRUint32 flags = nsIPresShell::FORCE_DRAW;
       rv = presShell->AddCanvasBackgroundColorItem(
              *aBuilder, childItems, subdocRootFrame ? subdocRootFrame : this,
              bounds, NS_RGBA(0,0,0,0), flags);
@@ -425,32 +396,6 @@ nsSubDocumentFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
 
     bool addedLayer = false;
 
-#ifdef MOZ_IPC
-    // Make a scrollable layer in the child process so it can be manipulated
-    // with transforms in the parent process.
-    if (XRE_GetProcessType() == GeckoProcessType_Content) {
-      nsIScrollableFrame* scrollFrame = presShell->GetRootScrollFrameAsScrollable();
-
-      if (scrollFrame) {
-        NS_ASSERTION(subdocRootFrame, "Root scroll frame should be non-null");
-        nsRect scrollRange = scrollFrame->GetScrollRange();
-        
-        // Since making new layers is expensive, only use nsDisplayScrollLayer
-        // if the area is scrollable.
-        if (scrollRange.width != 0 || scrollRange.height != 0) {
-          addedLayer = true;
-          nsDisplayScrollLayer* layerItem = new (aBuilder) nsDisplayScrollLayer(
-            aBuilder,
-            &childItems,
-            subdocRootScrollFrame,
-            subdocRootFrame
-          );
-          childItems.AppendToTop(layerItem);
-        }
-      }
-    }
-#endif
-
     if (subdocRootFrame && parentAPD != subdocAPD) {
       NS_WARN_IF_FALSE(!addedLayer,
                        "Two container layers have been added. "
@@ -470,10 +415,17 @@ nsSubDocumentFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
       childItems.AppendToTop(layerItem);
     }
 
+    nsDisplayList list;
     // Clip children to the child root frame's rectangle
-    rv = aLists.Content()->AppendNewToTop(
+    rv = list.AppendNewToTop(
         new (aBuilder) nsDisplayClip(aBuilder, this, &childItems,
                                      subdocBoundsInParentUnits));
+
+    if (mIsInline) {
+      WrapReplacedContentForBorderRadius(aBuilder, &list, aLists);
+    } else {
+      aLists.Content()->AppendToTop(&list);
+    }
   }
   // delete childItems in case of OOM
   childItems.DeleteAll();
@@ -535,7 +487,7 @@ nsSubDocumentFrame::GetType() const
 }
 
 /* virtual */ nscoord
-nsSubDocumentFrame::GetMinWidth(nsIRenderingContext *aRenderingContext)
+nsSubDocumentFrame::GetMinWidth(nsRenderingContext *aRenderingContext)
 {
   nscoord result;
   DISPLAY_MIN_WIDTH(this, result);
@@ -551,7 +503,7 @@ nsSubDocumentFrame::GetMinWidth(nsIRenderingContext *aRenderingContext)
 }
 
 /* virtual */ nscoord
-nsSubDocumentFrame::GetPrefWidth(nsIRenderingContext *aRenderingContext)
+nsSubDocumentFrame::GetPrefWidth(nsRenderingContext *aRenderingContext)
 {
   nscoord result;
   DISPLAY_PREF_WIDTH(this, result);
@@ -587,7 +539,7 @@ nsSubDocumentFrame::GetIntrinsicRatio()
 }
 
 /* virtual */ nsSize
-nsSubDocumentFrame::ComputeAutoSize(nsIRenderingContext *aRenderingContext,
+nsSubDocumentFrame::ComputeAutoSize(nsRenderingContext *aRenderingContext,
                                     nsSize aCBSize, nscoord aAvailableWidth,
                                     nsSize aMargin, nsSize aBorder,
                                     nsSize aPadding, PRBool aShrinkWrap)
@@ -605,7 +557,7 @@ nsSubDocumentFrame::ComputeAutoSize(nsIRenderingContext *aRenderingContext,
 
 
 /* virtual */ nsSize
-nsSubDocumentFrame::ComputeSize(nsIRenderingContext *aRenderingContext,
+nsSubDocumentFrame::ComputeSize(nsRenderingContext *aRenderingContext,
                                 nsSize aCBSize, nscoord aAvailableWidth,
                                 nsSize aMargin, nsSize aBorder, nsSize aPadding,
                                 PRBool aShrinkWrap)
@@ -747,6 +699,17 @@ nsSubDocumentFrame::AttributeChanged(PRInt32 aNameSpaceID,
         FrameNeedsReflow(rootFrame, nsIPresShell::eResize, NS_FRAME_IS_DIRTY);
     }
   }
+  else if (aAttribute == nsGkAtoms::marginwidth ||
+           aAttribute == nsGkAtoms::marginheight) {
+
+    // Retrieve the attributes
+    nsIntSize margins = GetMarginAttributes();
+
+    // Notify the frameloader
+    nsRefPtr<nsFrameLoader> frameloader = FrameLoader();
+    if (frameloader)
+      frameloader->MarginsChanged(margins.width, margins.height);
+  }
   else if (aAttribute == nsGkAtoms::type) {
     if (!mFrameLoader) 
       return NS_OK;
@@ -755,6 +718,11 @@ nsSubDocumentFrame::AttributeChanged(PRInt32 aNameSpaceID,
       return NS_OK;
     }
 
+    if (mFrameLoader->GetRemoteBrowser()) {
+      // TODO: Implement ContentShellAdded for remote browsers (bug 658304)
+      return NS_OK;
+    }
+
     // Note: This logic duplicates a lot of logic in
     // nsFrameLoader::EnsureDocShell.  We should fix that.
 
@@ -990,7 +958,7 @@ EndSwapDocShellsForDocument(nsIDocument* aDocument, void*)
   NS_PRECONDITION(aDocument, "");
 
   // Our docshell and view trees have been updated for the new hierarchy.
-  // Now also update all nsThebesDeviceContext::mWidget to that of the
+  // Now also update all nsDeviceContext::mWidget to that of the
   // container view in the new hierarchy.
   nsCOMPtr<nsISupports> container = aDocument->GetContainer();
   nsCOMPtr<nsIDocShell> ds = do_QueryInterface(container);
@@ -1002,7 +970,7 @@ EndSwapDocShellsForDocument(nsIDocument* aDocument, void*)
       if (dv) {
         nsCOMPtr<nsPresContext> pc;
         dv->GetPresContext(getter_AddRefs(pc));
-        nsIDeviceContext* dc = pc ? pc->DeviceContext() : nsnull;
+        nsDeviceContext* dc = pc ? pc->DeviceContext() : nsnull;
         if (dc) {
           nsIView* v = dv->FindContainerView();
           dc->Init(v ? v->GetNearestWidget(nsnull) : nsnull);
@@ -1111,12 +1079,10 @@ nsSubDocumentFrame::ObtainIntrinsicSizeFrame()
       }
     }
 
-#ifdef MOZ_SVG
     if (subDocRoot && subDocRoot->GetContent() &&
         subDocRoot->GetContent()->NodeInfo()->Equals(nsGkAtoms::svg, kNameSpaceID_SVG)) {
       return subDocRoot; // SVG documents have an intrinsic size
     }
-#endif
   }
   return nsnull;
 }
