@@ -834,9 +834,9 @@ nsAnimationManager::GetAnimationRule(mozilla::dom::Element* aElement,
     return nsnull;
   }
 
-  NS_ABORT_IF_FALSE(ea->mStyleRuleRefreshTime ==
-                      mPresContext->RefreshDriver()->MostRecentRefresh(),
-                    "should already have refreshed style rule");
+  NS_WARN_IF_FALSE(ea->mStyleRuleRefreshTime ==
+                     mPresContext->RefreshDriver()->MostRecentRefresh(),
+                   "should already have refreshed style rule");
 
   if (mPresContext->IsProcessingRestyles() &&
       !mPresContext->IsProcessingAnimationStyleChange()) {
