@@ -50,7 +50,7 @@
 #include "nsINameSpaceManager.h"
 #include "nsGkAtoms.h"
 #include "nsMenuPopupFrame.h"
-
+#include "nsClientRect.h"
 
 class nsPopupBoxObject : public nsBoxObject,
                          public nsIPopupBoxObject
@@ -124,24 +124,27 @@ nsPopupBoxObject::OpenPopup(nsIDOMElement* aAnchorElement,
                             const nsAString& aPosition,
                             PRInt32 aXPos, PRInt32 aYPos,
                             PRBool aIsContextMenu,
-                            PRBool aAttributesOverride)
+                            PRBool aAttributesOverride,
+                            nsIDOMEvent* aTriggerEvent)
 {
   nsXULPopupManager* pm = nsXULPopupManager::GetInstance();
   if (pm && mContent) {
     nsCOMPtr<nsIContent> anchorContent(do_QueryInterface(aAnchorElement));
     pm->ShowPopup(mContent, anchorContent, aPosition, aXPos, aYPos,
-                  aIsContextMenu, aAttributesOverride, PR_FALSE, nsnull);
+                  aIsContextMenu, aAttributesOverride, PR_FALSE, aTriggerEvent);
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPopupBoxObject::OpenPopupAtScreen(PRInt32 aXPos, PRInt32 aYPos, PRBool aIsContextMenu)
+nsPopupBoxObject::OpenPopupAtScreen(PRInt32 aXPos, PRInt32 aYPos,
+                                    PRBool aIsContextMenu,
+                                    nsIDOMEvent* aTriggerEvent)
 {
   nsXULPopupManager* pm = nsXULPopupManager::GetInstance();
   if (pm && mContent)
-    pm->ShowPopupAtScreen(mContent, aXPos, aYPos, aIsContextMenu, nsnull);
+    pm->ShowPopupAtScreen(mContent, aXPos, aYPos, aIsContextMenu, aTriggerEvent);
   return NS_OK;
 }
 
@@ -232,6 +235,9 @@ nsPopupBoxObject::EnableKeyboardNavigator(PRBool aEnableKeyboardNavigator)
 NS_IMETHODIMP
 nsPopupBoxObject::GetPopupState(nsAString& aState)
 {
+  // set this here in case there's no frame for the popup
+  aState.AssignLiteral("closed");
+
   nsMenuPopupFrame *menuPopupFrame = GetMenuPopupFrame();
   if (menuPopupFrame) {
     switch (menuPopupFrame->PopupState()) {
@@ -247,7 +253,6 @@ nsPopupBoxObject::GetPopupState(nsAString& aState)
         aState.AssignLiteral("hiding");
         break;
       case ePopupClosed:
-        aState.AssignLiteral("closed");
         break;
       default:
         NS_NOTREACHED("Bad popup state");
@@ -258,6 +263,66 @@ nsPopupBoxObject::GetPopupState(nsAString& aState)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPopupBoxObject::GetTriggerNode(nsIDOMNode** aTriggerNode)
+{
+  *aTriggerNode = nsnull;
+
+  nsIContent* triggerContent = nsMenuPopupFrame::GetTriggerContent(GetMenuPopupFrame());
+  if (triggerContent)
+    CallQueryInterface(triggerContent, aTriggerNode);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPopupBoxObject::GetAnchorNode(nsIDOMElement** aAnchor)
+{
+  *aAnchor = nsnull;
+
+  nsMenuPopupFrame *menuPopupFrame = GetMenuPopupFrame();
+  if (!menuPopupFrame)
+    return NS_OK;
+
+  nsIContent* anchor = menuPopupFrame->GetAnchor();
+  if (anchor)
+    CallQueryInterface(anchor, aAnchor);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPopupBoxObject::GetOuterScreenRect(nsIDOMClientRect** aRect)
+{
+  nsClientRect* rect = new nsClientRect();
+  if (!rect)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*aRect = rect);
+
+  nsMenuPopupFrame *menuPopupFrame = GetMenuPopupFrame();
+  if (!menuPopupFrame)
+    return NS_OK;
+
+  // Return an empty rectangle if the popup is not open.
+  nsPopupState state = menuPopupFrame->PopupState();
+  if (state != ePopupOpen && state != ePopupOpenAndVisible)
+    return NS_OK;
+
+  nsIView* view = menuPopupFrame->GetView();
+  if (view) {
+    nsIWidget* widget = view->GetWidget();
+    if (widget) {
+      nsIntRect screenRect;
+      widget->GetScreenBounds(screenRect);
+
+      PRInt32 pp = menuPopupFrame->PresContext()->AppUnitsPerDevPixel();
+      rect->SetLayoutRect(screenRect.ToAppUnits(pp));
+    }
+  }
+
+  return NS_OK;
+}
 
 // Creation Routine ///////////////////////////////////////////////////////////////////////
 
