@@ -248,6 +248,9 @@ nsMozIconURI::SetSpec(const nsACString &aSpec)
   if (!strncmp("//stock/", iconPath.get(), 8))
   {
     mStockIcon.Assign(Substring(iconPath, 8));
+    // An icon identifier must always be specified.
+    if (mStockIcon.IsEmpty())
+      return NS_ERROR_MALFORMED_URI;
     return NS_OK;
   }
 
@@ -381,6 +384,19 @@ nsMozIconURI::SetPath(const nsACString &aPath)
   return NS_ERROR_FAILURE;
 }
 
+NS_IMETHODIMP
+nsMozIconURI::GetRef(nsACString &aRef)
+{
+  aRef.Truncate();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMozIconURI::SetRef(const nsACString &aRef)
+{
+  return NS_ERROR_FAILURE;
+}
+
 NS_IMETHODIMP
 nsMozIconURI::Equals(nsIURI *other, PRBool *result)
 {
@@ -399,6 +415,14 @@ nsMozIconURI::Equals(nsIURI *other, PRBool *result)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMozIconURI::EqualsExceptRef(nsIURI *other, PRBool *result)
+{
+  // GetRef/SetRef not supported by nsMozIconURI, so
+  // EqualsExceptRef() is the same as Equals().
+  return Equals(other, result);
+}
+
 NS_IMETHODIMP
 nsMozIconURI::SchemeIs(const char *i_Scheme, PRBool *o_Equals)
 {
@@ -425,9 +449,6 @@ nsMozIconURI::Clone(nsIURI **result)
   }
 
   nsMozIconURI *uri = new nsMozIconURI();
-  if (!uri)
-    return NS_ERROR_OUT_OF_MEMORY;
- 
   newIconURL.swap(uri->mIconURL);
   uri->mSize = mSize;
   uri->mContentType = mContentType;
@@ -440,6 +461,14 @@ nsMozIconURI::Clone(nsIURI **result)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMozIconURI::CloneIgnoringRef(nsIURI **result)
+{
+  // GetRef/SetRef not supported by nsMozIconURI, so
+  // CloneIgnoringRef() is the same as Clone().
+  return Clone(result);
+}
+
 NS_IMETHODIMP
 nsMozIconURI::Resolve(const nsACString &relativePath, nsACString &result)
 {
