@@ -306,7 +306,7 @@ nsPNGDecoder::WriteInternal(const char *aBuffer, PRUint32 aCount)
       return;
 
     // Read data into our header buffer
-    PRUint32 bytesToRead = PR_MIN(aCount, BYTES_NEEDED_FOR_DIMENSIONS -
+    PRUint32 bytesToRead = NS_MIN(aCount, BYTES_NEEDED_FOR_DIMENSIONS -
                                   mHeaderBytesRead);
     memcpy(mHeaderBuf + mHeaderBytesRead, aBuffer, bytesToRead);
     mHeaderBytesRead += bytesToRead;
@@ -386,13 +386,24 @@ PNGGetColorProfile(png_structp png_ptr, png_infop info_ptr,
   // First try to see if iCCP chunk is present
   if (png_get_valid(png_ptr, info_ptr, PNG_INFO_iCCP)) {
     png_uint_32 profileLen;
+#if (PNG_LIBPNG_VER < 10500)
     char *profileData, *profileName;
+#else
+    png_bytep profileData;
+    png_charp profileName;
+#endif
     int compression;
 
     png_get_iCCP(png_ptr, info_ptr, &profileName, &compression,
                  &profileData, &profileLen);
 
-    profile = qcms_profile_from_memory(profileData, profileLen);
+    profile = qcms_profile_from_memory(
+#if (PNG_LIBPNG_VER < 10500)
+                                       profileData,
+#else
+                                       (char *)profileData,
+#endif
+                                       profileLen);
     if (profile) {
       PRUint32 profileSpace = qcms_profile_get_color_space(profile);
 
@@ -501,6 +512,11 @@ nsPNGDecoder::info_callback(png_structp png_ptr, png_infop info_ptr)
 
   // Post our size to the superclass
   decoder->PostSize(width, height);
+  if (decoder->HasError()) {
+    // Setting the size lead to an error; this can happen when for example
+    // a multipart channel sends an image of a different size.
+    longjmp(png_jmpbuf(decoder->mPNG), 1);
+  }
 
   if (color_type == PNG_COLOR_TYPE_PALETTE)
     png_set_expand(png_ptr);
