@@ -52,6 +52,8 @@
 #include "imgRequest.h"
 #include "nsIObserverService.h"
 #include "nsIChannelPolicy.h"
+#include "nsIProgressEventSink.h"
+#include "nsIChannel.h"
 
 #ifdef LOADER_THREADSAFE
 #include "prlock.h"
@@ -62,12 +64,11 @@ class imgRequestProxy;
 class imgIRequest;
 class imgIDecoderObserver;
 class nsILoadGroup;
-class nsIPrefBranch;
 
 class imgCacheEntry
 {
 public:
-  imgCacheEntry(imgRequest *request, PRBool mustValidateIfExpired = PR_FALSE);
+  imgCacheEntry(imgRequest *request);
   ~imgCacheEntry();
 
   nsrefcnt AddRef()
@@ -124,13 +125,13 @@ public:
     Touch();
   }
 
-  PRBool GetMustValidateIfExpired() const
+  PRBool GetMustValidate() const
   {
-    return mMustValidateIfExpired;
+    return mMustValidate;
   }
-  void SetMustValidateIfExpired(PRBool aValidate)
+  void SetMustValidate(PRBool aValidate)
   {
-    mMustValidateIfExpired = aValidate;
+    mMustValidate = aValidate;
     Touch();
   }
 
@@ -179,7 +180,7 @@ private: // data
   PRInt32 mTouchedTime;
   PRInt32 mExpiryTime;
   nsExpirationState mExpirationState;
-  PRPackedBool mMustValidateIfExpired : 1;
+  PRPackedBool mMustValidate : 1;
   PRPackedBool mEvicted : 1;
   PRPackedBool mHasNoProxies : 1;
 };
@@ -323,7 +324,7 @@ private: // methods
                                     nsLoadFlags aLoadFlags, imgIRequest *aRequestProxy,
                                     imgIRequest **_retval);
 
-  void ReadAcceptHeaderPref(nsIPrefBranch *aBranch);
+  void ReadAcceptHeaderPref();
 
 
   typedef nsRefPtrHashtable<nsCStringHashKey, imgCacheEntry> imgCacheTable;
@@ -375,6 +376,33 @@ private:
   nsCOMPtr<nsIStreamListener> mDestListener;
 };
 
+/**
+ * A class that implements nsIProgressEventSink and forwards all calls to it to
+ * the original notification callbacks of the channel. Also implements
+ * nsIInterfaceRequestor and gives out itself for nsIProgressEventSink calls,
+ * and forwards everything else to the channel's notification callbacks.
+ */
+class nsProgressNotificationProxy : public nsIProgressEventSink
+                                  , public nsIChannelEventSink
+                                  , public nsIInterfaceRequestor
+{
+  public:
+    nsProgressNotificationProxy(nsIChannel* channel,
+                                imgIRequest* proxy)
+        : mImageRequest(proxy) {
+      channel->GetNotificationCallbacks(getter_AddRefs(mOriginalCallbacks));
+    }
+
+    NS_DECL_ISUPPORTS
+    NS_DECL_NSIPROGRESSEVENTSINK
+    NS_DECL_NSICHANNELEVENTSINK
+    NS_DECL_NSIINTERFACEREQUESTOR
+  private:
+    ~nsProgressNotificationProxy() {}
+
+    nsCOMPtr<nsIInterfaceRequestor> mOriginalCallbacks;
+    nsCOMPtr<nsIRequest> mImageRequest;
+};
 
 /**
  * validate checker
@@ -382,25 +410,36 @@ private:
 
 #include "nsCOMArray.h"
 
-class imgCacheValidator : public nsIStreamListener
+class imgCacheValidator : public nsIStreamListener,
+                          public nsIChannelEventSink,
+                          public nsIInterfaceRequestor,
+                          public nsIAsyncVerifyRedirectCallback
 {
 public:
-  imgCacheValidator(imgRequest *request, void *aContext);
+  imgCacheValidator(nsProgressNotificationProxy* progress, imgRequest *request, void *aContext);
   virtual ~imgCacheValidator();
 
   void AddProxy(imgRequestProxy *aProxy);
 
-  /* additional members */
   NS_DECL_ISUPPORTS
   NS_DECL_NSISTREAMLISTENER
   NS_DECL_NSIREQUESTOBSERVER
+  NS_DECL_NSICHANNELEVENTSINK
+  NS_DECL_NSIINTERFACEREQUESTOR
+  NS_DECL_NSIASYNCVERIFYREDIRECTCALLBACK
 
 private:
   nsCOMPtr<nsIStreamListener> mDestListener;
+  nsRefPtr<nsProgressNotificationProxy> mProgressProxy;
+  nsCOMPtr<nsIAsyncVerifyRedirectCallback> mRedirectCallback;
+  nsCOMPtr<nsIChannel> mRedirectChannel;
 
   nsRefPtr<imgRequest> mRequest;
   nsCOMArray<imgIRequest> mProxies;
 
+  nsRefPtr<imgRequest> mNewRequest;
+  nsRefPtr<imgCacheEntry> mNewEntry;
+
   void *mContext;
 
   static imgLoader sImgLoader;
