@@ -37,16 +37,18 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#ifndef imgRequestProxy_h__
+#define imgRequestProxy_h__
+
 #include "imgIRequest.h"
 #include "imgIDecoderObserver.h"
 #include "nsISecurityInfoProvider.h"
 
-#include "imgIContainer.h"
-#include "imgIDecoder.h"
 #include "nsIRequestObserver.h"
 #include "nsIChannel.h"
 #include "nsILoadGroup.h"
 #include "nsISupportsPriority.h"
+#include "nsITimedChannel.h"
 #include "nsCOMPtr.h"
 #include "nsAutoPtr.h"
 #include "nsThreadUtils.h"
@@ -61,7 +63,19 @@
     {0x8f, 0x65, 0x9c, 0x46, 0x2e, 0xe2, 0xbc, 0x95} \
 }
 
-class imgRequestProxy : public imgIRequest, public nsISupportsPriority, public nsISecurityInfoProvider
+class imgRequestNotifyRunnable;
+class imgStatusNotifyRunnable;
+
+namespace mozilla {
+namespace imagelib {
+class Image;
+} // namespace imagelib
+} // namespace mozilla
+
+class imgRequestProxy : public imgIRequest, 
+                        public nsISupportsPriority, 
+                        public nsISecurityInfoProvider,
+                        public nsITimedChannel
 {
 public:
   NS_DECL_ISUPPORTS
@@ -69,22 +83,65 @@ public:
   NS_DECL_NSIREQUEST
   NS_DECL_NSISUPPORTSPRIORITY
   NS_DECL_NSISECURITYINFOPROVIDER
+  // nsITimedChannel declared below
 
   imgRequestProxy();
   virtual ~imgRequestProxy();
 
-  // Callers to Init or ChangeOwner are required to call
-  // NotifyProxyListener on the request after (although not immediately
-  // after) doing so.
-  nsresult Init(imgRequest *request, nsILoadGroup *aLoadGroup, imgIDecoderObserver *aObserver);
+  // Callers to Init or ChangeOwner are required to call NotifyListener after
+  // (although not immediately after) doing so.
+  nsresult Init(imgRequest *request, nsILoadGroup *aLoadGroup,
+                mozilla::imagelib::Image* aImage,
+                nsIURI* aURI, imgIDecoderObserver *aObserver);
+
   nsresult ChangeOwner(imgRequest *aNewOwner); // this will change mOwner.  Do not call this if the previous
                                                // owner has already sent notifications out!
 
   void AddToLoadGroup();
   void RemoveFromLoadGroup(PRBool releaseLoadGroup);
 
+  inline PRBool HasObserver() const {
+    return mListener != nsnull;
+  }
+
+  void SetPrincipal(nsIPrincipal *aPrincipal);
+
+  // Asynchronously notify this proxy's listener of the current state of the
+  // image, and, if we have an imgRequest mOwner, any status changes that
+  // happen between the time this function is called and the time the
+  // notification is scheduled.
+  void NotifyListener();
+
+  // Synchronously notify this proxy's listener of the current state of the
+  // image. Only use this function if you are currently servicing an
+  // asynchronously-called function.
+  void SyncNotifyListener();
+
+  // Whether we want notifications from imgStatusTracker to be deferred until
+  // an event it has scheduled has been fired.
+  PRBool NotificationsDeferred() const
+  {
+    return mDeferNotifications;
+  }
+  void SetNotificationsDeferred(PRBool aDeferNotifications)
+  {
+    mDeferNotifications = aDeferNotifications;
+  }
+
+  // Setter for our |mImage| pointer, for imgRequest to use, once it
+  // instantiates an Image.
+  void SetImage(mozilla::imagelib::Image* aImage);
+
+  // Removes all animation consumers that were created with
+  // IncrementAnimationConsumers. This is necessary since we need
+  // to do it before the proxy itself is destroyed. See
+  // imgRequest::RemoveProxy
+  void ClearAnimationConsumers();
+
 protected:
-  friend class imgRequest;
+  friend class imgStatusTracker;
+  friend class imgStatusNotifyRunnable;
+  friend class imgRequestNotifyRunnable;
 
   class imgCancelRunnable;
   friend class imgCancelRunnable;
@@ -106,7 +163,9 @@ protected:
       nsresult mStatus;
   };
 
-
+  // The following notification functions are protected to ensure that (friend
+  // class) imgStatusTracker is the only class allowed to send us
+  // notifications.
 
   /* non-virtual imgIDecoderObserver methods */
   void OnStartDecode   ();
@@ -116,17 +175,15 @@ protected:
   void OnStopFrame     (PRUint32 aFrame);
   void OnStopContainer (imgIContainer *aContainer);
   void OnStopDecode    (nsresult status, const PRUnichar *statusArg); 
+  void OnDiscard       ();
 
   /* non-virtual imgIContainerObserver methods */
-  void FrameChanged(imgIContainer *aContainer, nsIntRect * aDirtyRect);
+  void FrameChanged(imgIContainer *aContainer,
+                    const nsIntRect *aDirtyRect);
 
-  /* non-virtual nsIRequestObserver (plus some) methods */
-  void OnStartRequest(nsIRequest *request, nsISupports *ctxt);
-  void OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresult statusCode, PRBool aLastPart); 
-
-  inline PRBool HasObserver() const {
-    return mListener != nsnull;
-  }
+  /* non-virtual sort-of-nsIRequestObserver methods */
+  void OnStartRequest();
+  void OnStopRequest(PRBool aLastPart);
 
   /* Finish up canceling ourselves */
   void DoCancel(nsresult status);
@@ -137,6 +194,23 @@ protected:
   void DoRemoveFromLoadGroup() {
     RemoveFromLoadGroup(PR_TRUE);
   }
+
+  // Return the imgStatusTracker associated with mOwner and/or mImage. It may
+  // live either on mOwner or mImage, depending on whether
+  //   (a) we have an mOwner at all
+  //   (b) whether mOwner has instantiated its image yet
+  imgStatusTracker& GetStatusTracker();
+
+  nsITimedChannel* TimedChannel()
+  {
+    if (!mOwner)
+      return nsnull;
+    return mOwner->mTimedChannel;
+  }
+
+public:
+  NS_FORWARD_SAFE_NSITIMEDCHANNEL(TimedChannel())
+
 private:
   friend class imgCacheValidator;
 
@@ -148,6 +222,17 @@ private:
   // means that imgRequest::mObservers will not have any stale pointers in it.
   nsRefPtr<imgRequest> mOwner;
 
+  // The URI of our request.
+  nsCOMPtr<nsIURI> mURI;
+
+  // The image we represent. Is null until data has been received, and is then
+  // set by imgRequest.
+  nsRefPtr<mozilla::imagelib::Image> mImage;
+
+  // Our principal. Is null until data has been received from the channel, and
+  // is then set by imgRequest.
+  nsCOMPtr<nsIPrincipal> mPrincipal;
+
   // mListener is only promised to be a weak ref (see imgILoader.idl),
   // but we actually keep a strong ref to it until we've seen our
   // first OnStopRequest.
@@ -155,7 +240,20 @@ private:
   nsCOMPtr<nsILoadGroup> mLoadGroup;
 
   nsLoadFlags mLoadFlags;
+  PRUint32    mLockCount;
+  PRUint32    mAnimationConsumers;
   PRPackedBool mCanceled;
   PRPackedBool mIsInLoadGroup;
   PRPackedBool mListenerIsStrongRef;
+  PRPackedBool mDecodeRequested;
+
+  // Whether we want to defer our notifications by the non-virtual Observer
+  // interfaces as image loads proceed.
+  PRPackedBool mDeferNotifications;
+
+  // We only want to send OnStartContainer once for each proxy, but we might
+  // get multiple OnStartContainer calls (e.g. from multipart/x-mixed-replace).
+  PRPackedBool mSentStartContainer;
 };
+
+#endif // imgRequestProxy_h__
