@@ -161,6 +161,10 @@ public:
     static void      SetDiskCacheEnabled(PRBool  enabled);
     // Sets the disk cache capacity (in kilobytes)
     static void      SetDiskCacheCapacity(PRInt32  capacity);
+    // Set max size for a disk-cache entry (in bytes). -1 disables this limit
+    static void      SetDiskCacheMaxEntrySize(PRInt32  maxSize);
+    // Set max size for a memory-cache entry (in bytes). -1 disables this limit
+    static void      SetMemoryCacheMaxEntrySize(PRInt32  maxSize);
 
     static void      SetOfflineCacheEnabled(PRBool  enabled);
     // Sets the offline cache capacity (in kilobytes)
