@@ -126,6 +126,24 @@ nsMIMEHeaderParamImpl::GetParameter(const nsACString& aHeaderVal,
     return NS_OK;
 }
 
+// remove backslash-encoded sequences from quoted-strings
+// modifies string in place, potentially shortening it
+void RemoveQuotedStringEscapes(char *src)
+{
+  char *dst = src;
+
+  for (char *c = src; *c; ++c)
+  {
+    if (c[0] == '\\' && c[1])
+    {
+      // skip backslash if not at end
+      ++c;
+    }
+    *dst++ = *c;
+  }
+  *dst = 0;
+}
+
 // moved almost verbatim from mimehdrs.cpp
 // char *
 // MimeHeaders_get_parameter (const char *header_value, const char *parm_name,
@@ -216,6 +234,8 @@ nsMIMEHeaderParamImpl::GetParameterInternal(const char *aHeaderValue,
     if (*str == '=') ++str;
     while (nsCRT::IsAsciiSpace(*str)) ++str;
 
+    PRBool needUnquote = PR_FALSE;
+    
     if (*str != '"')
     {
       // The value is a token, not a quoted string.
@@ -228,7 +248,9 @@ nsMIMEHeaderParamImpl::GetParameterInternal(const char *aHeaderValue,
     }
     else
     {
-      // The value is a quoted string. 
+      // The value is a quoted string.
+      needUnquote = PR_TRUE;
+      
       ++str;
       valueStart = str;
       for (valueEnd = str; *valueEnd; ++valueEnd)
@@ -251,8 +273,14 @@ nsMIMEHeaderParamImpl::GetParameterInternal(const char *aHeaderValue,
       //     line continuation -- jht 4/29/98 
       nsCAutoString tempStr(valueStart, valueEnd - valueStart);
       tempStr.StripChars("\r\n");
-      *aResult = ToNewCString(tempStr);
-      NS_ENSURE_TRUE(*aResult, NS_ERROR_OUT_OF_MEMORY);
+      char *res = ToNewCString(tempStr);
+      NS_ENSURE_TRUE(res, NS_ERROR_OUT_OF_MEMORY);
+      
+      if (needUnquote)
+        RemoveQuotedStringEscapes(res);
+            
+      *aResult = res;
+      
       // keep going, we may find a RFC 2231 encoded alternative
     }
     // case B, C, and D
