@@ -58,12 +58,10 @@
 #include "nsString.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
-#include "nsILocaleService.h"
 #include "nsIURI.h"
 #include "nsTime.h"
 #include "nsIProxyObjectManager.h"
 #include "nsCRT.h"
-#include "nsAutoLock.h"
 #include "nsUsageArrayHelper.h"
 #include "nsICertificateDialogs.h"
 #include "nsNSSCertHelper.h"
@@ -75,6 +73,8 @@
 #include "nsIObjectInputStream.h"
 #include "nsIProgrammingLanguage.h"
 
+#include "nsXULAppAPI.h"
+
 #include "nspr.h"
 extern "C" {
 #include "pk11func.h"
@@ -120,11 +120,29 @@ NS_IMPL_THREADSAFE_ISUPPORTS7(nsNSSCertificate, nsIX509Cert,
                                                 nsISerializable,
                                                 nsIClassInfo)
 
+/* static */
+nsNSSCertificate*
+nsNSSCertificate::Create(CERTCertificate *cert)
+{
+  if (GeckoProcessType_Default != XRE_GetProcessType()) {
+    NS_ERROR("Trying to initialize nsNSSCertificate in a non-chrome process!");
+    return nsnull;
+  }
+  if (cert)
+    return new nsNSSCertificate(cert);
+  else
+    return new nsNSSCertificate();
+}
+
 nsNSSCertificate*
 nsNSSCertificate::ConstructFromDER(char *certDER, int derLen)
 {
-  nsNSSCertificate* newObject = new nsNSSCertificate();
-  if (!newObject->InitFromDER(certDER, derLen)) {
+  // On non-chrome process prevent instantiation
+  if (GeckoProcessType_Default != XRE_GetProcessType())
+    return nsnull;
+
+  nsNSSCertificate* newObject = nsNSSCertificate::Create();
+  if (newObject && !newObject->InitFromDER(certDER, derLen)) {
     delete newObject;
     newObject = nsnull;
   }
@@ -162,6 +180,11 @@ nsNSSCertificate::nsNSSCertificate(CERTCertificate *cert) :
                                            mCertType(CERT_TYPE_NOT_YET_INITIALIZED),
                                            mCachedEVStatus(ev_status_unknown)
 {
+#if defined(DEBUG)
+  if (GeckoProcessType_Default != XRE_GetProcessType())
+    NS_ERROR("Trying to initialize nsNSSCertificate in a non-chrome process!");
+#endif
+
   nsNSSShutDownPreventionLock locker;
   if (isAlreadyShutDown())
     return;
@@ -176,6 +199,8 @@ nsNSSCertificate::nsNSSCertificate() :
   mCertType(CERT_TYPE_NOT_YET_INITIALIZED),
   mCachedEVStatus(ev_status_unknown)
 {
+  if (GeckoProcessType_Default != XRE_GetProcessType())
+    NS_ERROR("Trying to initialize nsNSSCertificate in a non-chrome process!");
 }
 
 nsNSSCertificate::~nsNSSCertificate()
@@ -597,7 +622,7 @@ nsNSSCertificate::GetWindowTitle(char * *aWindowTitle)
       }
     }
   } else {
-    NS_ASSERTION(0,"Somehow got nsnull for mCertificate in nsNSSCertificate.");
+    NS_ERROR("Somehow got nsnull for mCertificate in nsNSSCertificate.");
     *aWindowTitle = nsnull;
   }
   return NS_OK;
@@ -825,9 +850,11 @@ nsNSSCertificate::GetIssuer(nsIX509Cert * *aIssuer)
   CERTCertificate *issuer;
   issuer = CERT_FindCertIssuer(mCert, PR_Now(), certUsageSSLClient);
   if (issuer) {
-    nsCOMPtr<nsIX509Cert> cert = new nsNSSCertificate(issuer);
-    *aIssuer = cert;
-    NS_ADDREF(*aIssuer);
+    nsCOMPtr<nsIX509Cert> cert = nsNSSCertificate::Create(issuer);
+    if (cert) {
+      *aIssuer = cert;
+      NS_ADDREF(*aIssuer);
+    }
     CERT_DestroyCertificate(issuer);
   }
   return NS_OK;
@@ -882,7 +909,7 @@ nsNSSCertificate::GetChain(nsIArray **_rvChain)
        !CERT_LIST_END(node, nssChain);
        node = CERT_LIST_NEXT(node)) {
     PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("adding %s to chain\n", node->cert->nickname));
-    nsCOMPtr<nsIX509Cert> cert = new nsNSSCertificate(node->cert);
+    nsCOMPtr<nsIX509Cert> cert = nsNSSCertificate::Create(node->cert);
     array->AppendElement(cert, PR_FALSE);
   }
   *_rvChain = array;
@@ -984,9 +1011,10 @@ nsNSSCertificate::GetSerialNumber(nsAString &_serialNumber)
     return NS_ERROR_NOT_AVAILABLE;
 
   _serialNumber.Truncate();
-  nsXPIDLCString tmpstr; tmpstr.Adopt(CERT_Hexify(&mCert->serialNumber, 1));
-  if (tmpstr.get()) {
+  char *tmpstr = CERT_Hexify(&mCert->serialNumber, 1);
+  if (tmpstr) {
     _serialNumber = NS_ConvertASCIItoUTF16(tmpstr);
+    PORT_Free(tmpstr);
     return NS_OK;
   }
   return NS_ERROR_FAILURE;
@@ -1007,9 +1035,10 @@ nsNSSCertificate::GetSha1Fingerprint(nsAString &_sha1Fingerprint)
                mCert->derCert.data, mCert->derCert.len);
   fpItem.data = fingerprint;
   fpItem.len = SHA1_LENGTH;
-  nsXPIDLCString fpStr; fpStr.Adopt(CERT_Hexify(&fpItem, 1));
-  if (fpStr.get()) {
+  char *fpStr = CERT_Hexify(&fpItem, 1);
+  if (fpStr) {
     _sha1Fingerprint = NS_ConvertASCIItoUTF16(fpStr);
+    PORT_Free(fpStr);
     return NS_OK;
   }
   return NS_ERROR_FAILURE;
@@ -1030,9 +1059,10 @@ nsNSSCertificate::GetMd5Fingerprint(nsAString &_md5Fingerprint)
                mCert->derCert.data, mCert->derCert.len);
   fpItem.data = fingerprint;
   fpItem.len = MD5_LENGTH;
-  nsXPIDLCString fpStr; fpStr.Adopt(CERT_Hexify(&fpItem, 1));
-  if (fpStr.get()) {
+  char *fpStr = CERT_Hexify(&fpItem, 1);
+  if (fpStr) {
     _md5Fingerprint = NS_ConvertASCIItoUTF16(fpStr);
+    PORT_Free(fpStr);
     return NS_OK;
   }
   return NS_ERROR_FAILURE;
@@ -1259,6 +1289,15 @@ nsNSSCertificate::VerifyForUsage(PRUint32 usage, PRUint32 *verificationResult)
 
   NS_ENSURE_ARG(verificationResult);
 
+  nsresult nsrv;
+  nsCOMPtr<nsINSSComponent> inss = do_GetService(kNSSComponentCID, &nsrv);
+  if (!inss)
+    return nsrv;
+  nsRefPtr<nsCERTValInParamWrapper> survivingParams;
+  nsrv = inss->GetDefaultCERTValInParam(survivingParams);
+  if (NS_FAILED(nsrv))
+    return nsrv;
+  
   SECCertificateUsage nss_usage;
   
   switch (usage)
@@ -1315,10 +1354,21 @@ nsNSSCertificate::VerifyForUsage(PRUint32 usage, PRUint32 *verificationResult)
       return NS_ERROR_FAILURE;
   }
 
-  CERTCertDBHandle *defaultcertdb = CERT_GetDefaultCertDB();
-
-  if (CERT_VerifyCertificateNow(defaultcertdb, mCert, PR_TRUE, 
-                         nss_usage, NULL, NULL) == SECSuccess)
+  SECStatus verify_result;
+  if (!nsNSSComponent::globalConstFlagUsePKIXVerification) {
+    CERTCertDBHandle *defaultcertdb = CERT_GetDefaultCertDB();
+    verify_result = CERT_VerifyCertificateNow(defaultcertdb, mCert, PR_TRUE, 
+                                              nss_usage, NULL, NULL);
+  }
+  else {
+    CERTValOutParam cvout[1];
+    cvout[0].type = cert_po_end;
+    verify_result = CERT_PKIXVerifyCert(mCert, nss_usage,
+                                        survivingParams->GetRawPointerForNSS(),
+                                        cvout, NULL);
+  }
+  
+  if (verify_result == SECSuccess)
   {
     *verificationResult = VERIFIED_OK;
   }
@@ -1371,7 +1421,7 @@ nsNSSCertificate::VerifyForUsage(PRUint32 usage, PRUint32 *verificationResult)
 
 
 NS_IMETHODIMP
-nsNSSCertificate::GetUsagesArray(PRBool ignoreOcsp,
+nsNSSCertificate::GetUsagesArray(PRBool localOnly,
                                  PRUint32 *_verified,
                                  PRUint32 *_count,
                                  PRUnichar ***_usages)
@@ -1386,7 +1436,7 @@ nsNSSCertificate::GetUsagesArray(PRBool ignoreOcsp,
   const char *suffix = "";
   PRUint32 tmpCount;
   nsUsageArrayHelper uah(mCert);
-  rv = uah.GetUsagesArray(suffix, ignoreOcsp, max_usages, _verified, &tmpCount, tmpUsages);
+  rv = uah.GetUsagesArray(suffix, localOnly, max_usages, _verified, &tmpCount, tmpUsages);
   NS_ENSURE_SUCCESS(rv,rv);
   if (tmpCount > 0) {
     *_usages = (PRUnichar **)nsMemory::Alloc(sizeof(PRUnichar *) * tmpCount);
@@ -1426,7 +1476,7 @@ nsNSSCertificate::RequestUsagesArrayAsync(nsICertVerificationListener *aResultLi
 }
 
 NS_IMETHODIMP
-nsNSSCertificate::GetUsagesString(PRBool ignoreOcsp,
+nsNSSCertificate::GetUsagesString(PRBool localOnly,
                                   PRUint32   *_verified,
                                   nsAString &_usages)
 {
@@ -1440,7 +1490,7 @@ nsNSSCertificate::GetUsagesString(PRBool ignoreOcsp,
   const char *suffix = "_p";
   PRUint32 tmpCount;
   nsUsageArrayHelper uah(mCert);
-  rv = uah.GetUsagesArray(suffix, ignoreOcsp, max_usages, _verified, &tmpCount, tmpUsages);
+  rv = uah.GetUsagesArray(suffix, localOnly, max_usages, _verified, &tmpCount, tmpUsages);
   NS_ENSURE_SUCCESS(rv,rv);
   _usages.Truncate();
   for (PRUint32 i=0; i<tmpCount; i++) {
@@ -1639,12 +1689,12 @@ nsNSSCertList::AddCert(nsIX509Cert *aCert)
 
   cert = nssCert->GetCert();
   if (cert == nsnull) {
-    NS_ASSERTION(0,"Somehow got nsnull for mCertificate in nsNSSCertificate.");
+    NS_ERROR("Somehow got nsnull for mCertificate in nsNSSCertificate.");
     return NS_ERROR_FAILURE;
   }
 
   if (mCertList == nsnull) {
-    NS_ASSERTION(0,"Somehow got nsnull for mCertList in nsNSSCertList.");
+    NS_ERROR("Somehow got nsnull for mCertList in nsNSSCertList.");
     return NS_ERROR_FAILURE;
   }
   CERT_AddCertToListTail(mCertList,cert);
@@ -1662,12 +1712,12 @@ nsNSSCertList::DeleteCert(nsIX509Cert *aCert)
   CERTCertListNode *node;
 
   if (cert == nsnull) {
-    NS_ASSERTION(0,"Somehow got nsnull for mCertificate in nsNSSCertificate.");
+    NS_ERROR("Somehow got nsnull for mCertificate in nsNSSCertificate.");
     return NS_ERROR_FAILURE;
   }
 
   if (mCertList == nsnull) {
-    NS_ASSERTION(0,"Somehow got nsnull for mCertList in nsNSSCertList.");
+    NS_ERROR("Somehow got nsnull for mCertList in nsNSSCertList.");
     return NS_ERROR_FAILURE;
   }
 
@@ -1758,7 +1808,7 @@ nsNSSCertListEnumerator::GetNext(nsISupports **_retval)
     return NS_ERROR_FAILURE;
   }
 
-  nsCOMPtr<nsIX509Cert> nssCert = new nsNSSCertificate(node->cert);
+  nsCOMPtr<nsIX509Cert> nssCert = nsNSSCertificate::Create(node->cert);
   if (!nssCert) { 
     return NS_ERROR_OUT_OF_MEMORY;
   }
