@@ -105,27 +105,32 @@ function test() {
     }],
   }, {
     id: "addon4@tests.mozilla.org",
+    blocklistURL: "http://example.com/addon4@tests.mozilla.org",
     name: "Test add-on 4",
     _userDisabled: true,
     isActive: false,
     blocklistState: Ci.nsIBlocklistService.STATE_SOFTBLOCKED
   }, {
     id: "addon5@tests.mozilla.org",
+    blocklistURL: "http://example.com/addon5@tests.mozilla.org",
     name: "Test add-on 5",
     isActive: false,
     blocklistState: Ci.nsIBlocklistService.STATE_BLOCKED,
     appDisabled: true
   }, {
     id: "addon6@tests.mozilla.org",
+    blocklistURL: "http://example.com/addon6@tests.mozilla.org",
     name: "Test add-on 6",
     operationsRequiringRestart: AddonManager.OP_NEEDS_RESTART_NONE
   }, {
     id: "addon7@tests.mozilla.org",
+    blocklistURL: "http://example.com/addon7@tests.mozilla.org",
     name: "Test add-on 7",
     _userDisabled: true,
     isActive: false
   }, {
     id: "addon8@tests.mozilla.org",
+    blocklistURL: "http://example.com/addon8@tests.mozilla.org",
     name: "Test add-on 8",
     blocklistState: Ci.nsIBlocklistService.STATE_OUTDATED
   }]);
@@ -382,7 +387,7 @@ add_test(function() {
     is(get("detail-warning").textContent, "Test add-on 4 is known to cause security or stability issues.", "Warning message should be correct");
     is_element_visible(get("detail-warning-link"), "Warning link should be visible");
     is(get("detail-warning-link").value, "More Information", "Warning link text should be correct");
-    is(get("detail-warning-link").href, gBlocklistURL, "Warning link should be correct");
+    is(get("detail-warning-link").href, "http://example.com/addon4@tests.mozilla.org", "Warning link should be correct");
     is_element_hidden(get("detail-error"), "Error message should be hidden");
     is_element_hidden(get("detail-error-link"), "Error link should be hidden");
     is_element_hidden(get("detail-pending"), "Pending message should be hidden");
@@ -426,7 +431,7 @@ add_test(function() {
       is(get("detail-warning").textContent, "Test add-on 4 is known to cause security or stability issues.", "Warning message should be correct");
       is_element_visible(get("detail-warning-link"), "Warning link should be visible");
       is(get("detail-warning-link").value, "More Information", "Warning link text should be correct");
-      is(get("detail-warning-link").href, gBlocklistURL, "Warning link should be correct");
+      is(get("detail-warning-link").href, "http://example.com/addon4@tests.mozilla.org", "Warning link should be correct");
       is_element_hidden(get("detail-error"), "Error message should be hidden");
       is_element_hidden(get("detail-error-link"), "Error link should be hidden");
       is_element_hidden(get("detail-pending"), "Pending message should be hidden");
@@ -452,7 +457,7 @@ add_test(function() {
     is(get("detail-error").textContent, "Test add-on 5 has been disabled due to security or stability issues.", "Error message should be correct");
     is_element_visible(get("detail-error-link"), "Error link should be visible");
     is(get("detail-error-link").value, "More Information", "Error link text should be correct");
-    is(get("detail-error-link").href, gBlocklistURL, "Error link should be correct");
+    is(get("detail-error-link").href, "http://example.com/addon5@tests.mozilla.org", "Error link should be correct");
     is_element_hidden(get("detail-pending"), "Pending message should be hidden");
 
     run_next_test();
