@@ -8,7 +8,11 @@
 
 #include "nsUTF8Utils.h"
 
-#if defined(_MSC_VER) && defined(_M_IX86) && defined(XRE_WANT_DLL_BLOCKLIST)
+#ifndef XRE_DONT_PROTECT_DLL_LOAD
+#include "nsSetDllDirectory.h"
+#endif
+
+#if defined(_MSC_VER) && (defined(_M_IX86) || defined(_M_X64)) && defined(XRE_WANT_DLL_BLOCKLIST)
 #include "nsWindowsDllBlocklist.cpp"
 #else
 #undef XRE_WANT_DLL_BLOCKLIST
@@ -69,27 +73,13 @@ FreeAllocStrings(int argc, char **argv)
   delete [] argv;
 }
 
-#ifdef WINCE
-/** argc/argv are in/out parameters */
-void ExtractEnvironmentFromCL(int &argc, char **&argv)
-{
-  for (int x = argc - 1; x >= 0; x--) {
-    if (!strncmp(argv[x], "--environ:", 10)) {
-      char* key_val = strdup(argv[x]+10);
-      putenv(key_val);
-      free(key_val);
-      argc -= 1;
-      char *delete_argv = argv[x];
-      if (x < argc) /* if the current argument is not at the tail, shift following arguments. */
-        memcpy(&argv[x], &argv[x+1], (argc - x) * sizeof(char*));
-      delete [] delete_argv;
-    }
-  } 
-}
-#endif  
-
 int wmain(int argc, WCHAR **argv)
 {
+#ifndef XRE_DONT_PROTECT_DLL_LOAD
+  mozilla::SanitizeEnvironmentVariables();
+  mozilla::NS_SetDllDirectory(L"");
+#endif
+
 #ifdef XRE_WANT_DLL_BLOCKLIST
   SetupDllBlocklist();
 #endif
@@ -104,9 +94,6 @@ int wmain(int argc, WCHAR **argv)
       return 127;
     }
   }
-#ifdef WINCE
-  ExtractEnvironmentFromCL(argc, argvConverted);
-#endif
   argvConverted[argc] = NULL;
 
   // need to save argvConverted copy for later deletion.
