@@ -43,9 +43,9 @@
 #include "xpcom-private.h"
 
 //-----------------------------------------------------------------------------
-// XP_MACOSX or XP_BEOS
+// XP_MACOSX or ANDROID
 //-----------------------------------------------------------------------------
-#if defined(XP_BEOS) || defined(XP_MACOSX)
+#if defined(XP_MACOSX) || defined(ANDROID)
 
 #include "nsAString.h"
 #include "nsReadableUtils.h"
@@ -83,11 +83,13 @@ NS_ShutdownNativeCharsetUtils()
 
 #include <stdlib.h>   // mbtowc, wctomb
 #include <locale.h>   // setlocale
+#include "mozilla/Mutex.h"
 #include "nscore.h"
-#include "prlock.h"
 #include "nsAString.h"
 #include "nsReadableUtils.h"
 
+using namespace mozilla;
+
 //
 // choose a conversion library.  we used to use mbrtowc/wcrtomb under Linux,
 // but that doesn't work for non-BMP characters whether we use '-fshort-wchar'
@@ -307,14 +309,14 @@ private:
     static iconv_t gUnicodeToUTF8;
     static iconv_t gUTF8ToUnicode;
 #endif
-    static PRLock *gLock;
+    static Mutex  *gLock;
     static PRBool  gInitialized;
     static PRBool  gIsNativeUTF8;
 
     static void LazyInit();
 
-    static void Lock()   { if (gLock) PR_Lock(gLock);   }
-    static void Unlock() { if (gLock) PR_Unlock(gLock); }
+    static void Lock()   { if (gLock) gLock->Lock();   }
+    static void Unlock() { if (gLock) gLock->Unlock(); }
 };
 
 iconv_t nsNativeCharsetConverter::gNativeToUnicode = INVALID_ICONV_T;
@@ -325,7 +327,7 @@ iconv_t nsNativeCharsetConverter::gUTF8ToNative    = INVALID_ICONV_T;
 iconv_t nsNativeCharsetConverter::gUnicodeToUTF8   = INVALID_ICONV_T;
 iconv_t nsNativeCharsetConverter::gUTF8ToUnicode   = INVALID_ICONV_T;
 #endif
-PRLock *nsNativeCharsetConverter::gLock            = nsnull;
+Mutex  *nsNativeCharsetConverter::gLock            = nsnull;
 PRBool  nsNativeCharsetConverter::gInitialized     = PR_FALSE;
 PRBool  nsNativeCharsetConverter::gIsNativeUTF8    = PR_FALSE;
 
@@ -407,15 +409,14 @@ nsNativeCharsetConverter::LazyInit()
 void
 nsNativeCharsetConverter::GlobalInit()
 {
-    gLock = PR_NewLock();
-    NS_ASSERTION(gLock, "lock creation failed");
+    gLock = new Mutex("nsNativeCharsetConverter.gLock");
 }
 
 void
 nsNativeCharsetConverter::GlobalShutdown()
 {
     if (gLock) {
-        PR_DestroyLock(gLock);
+        delete gLock;
         gLock = nsnull;
     }
 
