@@ -44,6 +44,7 @@
 #define PROCESSMODEL_WINAPI
 #endif
 
+#include "mozilla/Mutex.h"
 #include "nsIProcess.h"
 #include "nsIFile.h"
 #include "nsIThread.h"
@@ -89,7 +90,7 @@ private:
                       PRBool holdWeak, PRBool argsUTF8);
 
   PRThread* mThread;
-  PRLock* mLock;
+  mozilla::Mutex mLock;
   PRBool mShutdown;
 
   nsCOMPtr<nsIFile> mExecutable;
