@@ -53,7 +53,6 @@
 #include "prio.h"
 #include "prenv.h"
 #include "nsCRT.h"
-#include "nsAutoLock.h"
 #include "nsThreadUtils.h"
 #include "nsIObserverService.h"
 #include "mozilla/Services.h"
@@ -75,14 +74,7 @@
 #include <signal.h>
 #endif
 
-#ifdef WINCE
-#include <windows.h> // for MultiByteToWideChar
-#include "prmem.h"
-#define SHELLEXECUTEINFOW SHELLEXECUTEINFO
-#define SEE_MASK_FLAG_DDEWAIT 0
-#define SEE_MASK_NO_CONSOLE 0
-#define ShellExecuteExW ShellExecuteEx
-#endif
+using namespace mozilla;
 
 #ifdef XP_MACOSX
 cpu_type_t pref_cpu_types[2] = {
@@ -105,7 +97,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS2(nsProcess, nsIProcess,
 //Constructor
 nsProcess::nsProcess()
     : mThread(nsnull)
-    , mLock(PR_NewLock())
+    , mLock("nsProcess.mLock")
     , mShutdown(PR_FALSE)
     , mPid(-1)
     , mObserver(nsnull)
@@ -120,7 +112,6 @@ nsProcess::nsProcess()
 //Destructor
 nsProcess::~nsProcess()
 {
-    PR_DestroyLock(mLock);
 }
 
 NS_IMETHODIMP
@@ -276,7 +267,7 @@ void PR_CALLBACK nsProcess::Monitor(void *arg)
 
     // Lock in case Kill or GetExitCode are called during this
     {
-        nsAutoLock lock(process->mLock);
+        MutexAutoLock lock(process->mLock);
         CloseHandle(process->mProcess);
         process->mProcess = NULL;
         process->mExitValue = exitCode;
@@ -303,7 +294,7 @@ void PR_CALLBACK nsProcess::Monitor(void *arg)
 
     // Lock in case Kill or GetExitCode are called during this
     {
-        nsAutoLock lock(process->mLock);
+        MutexAutoLock lock(process->mLock);
 #if !defined(XP_MACOSX)
         process->mProcess = nsnull;
 #endif
@@ -544,13 +535,11 @@ nsProcess::RunProcess(PRBool blocking, char **my_argv, nsIObserver* observer,
     mProcess = PR_CreateProcess(my_argv[0], my_argv, NULL, NULL);
     if (!mProcess)
         return NS_ERROR_FAILURE;
-#if !defined WINCE
     struct MYProcess {
         PRUint32 pid;
     };
     MYProcess* ptrProc = (MYProcess *) mProcess;
     mPid = ptrProc->pid;
-#endif
 #endif
 
     NS_ADDREF_THIS();
@@ -606,7 +595,7 @@ nsProcess::Kill()
         return NS_ERROR_FAILURE;
 
     {
-        nsAutoLock lock(mLock);
+        MutexAutoLock lock(mLock);
 #if defined(PROCESSMODEL_WINAPI)
         if (TerminateProcess(mProcess, NULL) == 0)
             return NS_ERROR_FAILURE;
@@ -633,7 +622,7 @@ nsProcess::Kill()
 NS_IMETHODIMP
 nsProcess::GetExitValue(PRInt32 *aExitValue)
 {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
 
     *aExitValue = mExitValue;
     
@@ -655,7 +644,7 @@ nsProcess::Observe(nsISupports* subject, const char* topic, const PRUnichar* dat
     mObserver = nsnull;
     mWeakObserver = nsnull;
 
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     mShutdown = PR_TRUE;
 
     return NS_OK;
